import numpy as np
import sys

def first_derivative(f_in,x_in):
    x = x_in.flatten()
    f = f_in.flatten()
    dx = x[1]-x[0]
    dx1 = x[2] - x[1]
    if abs(dx - dx1) > 1.E-10:
        print(dx, dx1)
        sys.exit("x grid must be uniform")
    dfdx = np.empty(len(f))
    for i in range(len(f)):
        if i == 0:
            dfdx[i] = (f[i+1] - f[i])/dx
        elif i == 1 or i == len(f)-2:
            dfdx[i] = (f[i+1]-f[i-1])/2./dx
        elif i == len(f)-1:
            dfdx[i] = (f[i] - f[i-1])/dx
        else:
            dfdx[i] = (-f[i+2]+8.*f[i+1]-8.*f[i-1]+f[i-2])/12./dx
	
    return dfdx




#! /usr/bin/python

from pylab import *
from sys import argv,exit,stdout
import matplotlib.pyplot as plt
from scipy.interpolate import UnivariateSpline
from scipy import interpolate
import numpy as np
from .finite_differences_x import *
from .interp import *

def read_EFIT(EFIT_file_name):
    EFITdict = {}
    f = open(EFIT_file_name,'r')
    eqdsk=f.readlines()
    line1=eqdsk[0].split()
    print(line1)
    if len(line1) == 2:
        nwnh = eqdsk[0].split()[1]
        nw = int(nwnh[0:3])
        nh = int(nwnh[3:7])
    else:
        nw = int(line1[-2])
        nh = int(line1[-1])

    EFITdict['nw'] = nw    #number of grid for Rgrid
    EFITdict['nh'] = nh    #number of grid for Zgrid

    entrylength=16
    #note: here rmin is rleft from EFIT
    #print(len(eqdsk[1])/entrylength) is not integer
    try:
        rdim,zdim,rcentr,rleft,zmid = \
            [float(eqdsk[1][j*entrylength:(j+1)*entrylength]) \
            for j in range(len(eqdsk[1])//entrylength)]
    except:
        entrylength=15
        try:
            rdim,zdim,rcentr,rleft,zmid = \
                [float(eqdsk[1][j*entrylength:(j+1)*entrylength]) \
                for j in range(len(eqdsk[1])//entrylength)]
        except:
            exit('Error reading EQDSK file, please check format!')

    rmaxis,zmaxis,simag,sibry,bcentr = \
        [float(eqdsk[2][j*entrylength:(j+1)*entrylength]) \
        for j in range(len(eqdsk[2])//entrylength)]
    current,simag2,xdum,rmaxis2,xdum = \
        [float(eqdsk[3][j*entrylength:(j+1)*entrylength]) \
        for j in range(len(eqdsk[3])//entrylength)]
    zmaxis2,xdum,sibry2,xdum,xdum = \
        [float(eqdsk[4][j*entrylength:(j+1)*entrylength]) \
        for j in range(len(eqdsk[4])//entrylength)]

    EFITdict['rdim'] = rdim
    EFITdict['zdim'] = zdim
    EFITdict['rcentr'] = rcentr
    EFITdict['rleft'] = rleft
    EFITdict['zmid'] = zmid
    EFITdict['rmaxis'] = rmaxis    # R of magnetic axis (m)
    EFITdict['zmaxis'] = zmaxis    # Z of magnetic axis (m)
    EFITdict['simag'] = simag    # poloidal flux at magnetic axis
    EFITdict['sibry'] = sibry    # poloidal flux at plasma boundary
    EFITdict['bcentr'] = bcentr    # vacuum toroidal magnetic field in Telsa
    EFITdict['current'] = current    # plasma current in Ampere

    print('EFIT file Resolution: %d x %d' %(EFITdict['nw'],EFITdict['nh']))
    print('Horizontal dimension(m): %10.4f' %EFITdict['rdim'])
    print('Vertical dimension(m): %10.4f' %EFITdict['zdim'])
    print('Minimum R of rectangular grid: %10.4f' %EFITdict['rleft'])
    print('(R, Z) of magnetic axis: (%10.4f, %10.4f)' %(EFITdict['rmaxis'],EFITdict['zmaxis']))
    print('poloidal flux at magnetic axis in Weber/rad: %10.4f' %EFITdict['simag'])
    print('poloidal flux at the plasma boundary in Weber/rad: %10.4f' %EFITdict['sibry'])
    print('Vacuum toroidal magnetic field at R = %10.4f: %10.4f Tesla' %(EFITdict['rcentr'],EFITdict['bcentr']))
    print('Z of center of rectangular grid: %10.4f' %EFITdict['zmid'])
    print('Plasma current: %10.4f Ampere' %EFITdict['current'])

    Rgrid = np.linspace(0, 1, nw, endpoint = True) * rdim + rleft
    Zgrid = np.linspace(0, 1, nh, endpoint = True) * zdim + (zmid - zdim/2.)
    EFITdict['Rgrid'] = Rgrid    # Rgrid of psi(Z, R)
    EFITdict['Zgrid'] = Zgrid    # Zgrid of psi(Z, R)

    Fpol = empty(nw, dtype = float)
    Pres = empty(nw, dtype = float)
    FFprime = empty(nw, dtype = float)
    Pprime = empty(nw, dtype = float)
    qpsi = empty(nw, dtype = float)
    jtor = empty(nw, dtype = float)

    # psi_pol is written on uniform (R,Z) grid (res=nw(R)*nh(Z))
    psirz_1d = empty(nw * nh, dtype = float)
    
    start_line = 5
    wordsInLine = 5
    lines=range(nw//wordsInLine)
    if nw%wordsInLine!=0: lines=range(nw//wordsInLine+1)
    for i in lines:
        n_entries=len(eqdsk[i+start_line])//entrylength
        Fpol[i*wordsInLine:i*wordsInLine+n_entries] = \
            [float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) \
            for j in range(n_entries)]
    start_line=i+start_line+1
    EFITdict['Fpol'] = Fpol    # poloidal current function F = R * Btor on psipn grid

    for i in lines:
        n_entries=len(eqdsk[i+start_line])//entrylength
        Pres[i*wordsInLine:i*wordsInLine+n_entries] = \
            [float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) \
            for j in range(n_entries)]
    start_line=i+start_line+1
    EFITdict['Pres'] = Pres    # plasma pressure in N / m^2 on psipn grid

    for i in lines:
        n_entries=len(eqdsk[i+start_line])//entrylength
        FFprime[i*wordsInLine:i*wordsInLine+n_entries] = \
            [float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) \
            for j in range(n_entries)]
    start_line=i+start_line+1
    EFITdict['FFprime'] = FFprime    # FFprime on psipn grid

    for i in lines:
        n_entries=len(eqdsk[i+start_line])//entrylength
        Pprime[i*wordsInLine:i*wordsInLine+n_entries] = \
            [float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) \
            for j in range(n_entries)]
    start_line=i+start_line+1
    EFITdict['Pprime'] = Pprime    # Pprime on psipn grid

    lines_twod=range(nw*nh//wordsInLine)
    if nw*nh%wordsInLine!=0: lines_twod=range(nw*nh//wordsInLine+1)
    for i in lines_twod:
        n_entries=len(eqdsk[i+start_line])//entrylength
        psirz_1d[i*wordsInLine:i*wordsInLine+n_entries] = \
            [float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) \
            for j in range(n_entries)]
    start_line=i+start_line+1

    psirz=psirz_1d.reshape(nh,nw)
    EFITdict['psirz'] = psirz    # poloidal flux on the rectangular grid (Rgrid, Zgrid)

    for i in lines:
        n_entries=len(eqdsk[i+start_line])//entrylength
        qpsi[i*wordsInLine:i*wordsInLine+n_entries] = \
            [float(eqdsk[i+start_line][j*entrylength:(j+1)*entrylength]) \
            for j in range(n_entries)]
    start_line=i+start_line+1
    EFITdict['qpsi'] = qpsi    # safety factor on psipn grid

    # even grid of psi_pol, on which all 1D fields are defined
    psipn = np.linspace(0., 1., nw)
    EFITdict['psipn'] = psipn    # uniform psipn grid

    interpol_order = 3 
    psip = psipn * (sibry - simag) + simag
    q_spl_psi = UnivariateSpline(psip, qpsi, k=interpol_order, s=1e-5)
    psi_pol_fine = linspace(psip[0], psip[-1], nw*10)
    psi_tor_fine = empty((nw*10),dtype=float)
    psi_tor_fine[0] = 0.
    for i in range(1, nw * 10):
        x = psi_pol_fine[:i+1]
        y = q_spl_psi(x)
        psi_tor_fine[i] = np.trapz(y,x)

    rhot_n_fine = np.sqrt(psi_tor_fine/(psi_tor_fine[-1]-psi_tor_fine[0]))
    rho_tor_spl = UnivariateSpline(psi_pol_fine, rhot_n_fine, k=interpol_order, s=1e-5)
    rhotn = rho_tor_spl(psip)
    EFITdict['rhotn'] = rhotn    # square root of toroidal flux on psipn grid

    Z0_ind = np.argmin(abs(Zgrid - zmaxis))
    R0_ind = np.argmin(abs(Rgrid - rmaxis - 0.02))
    R_obmp = Rgrid[R0_ind:]
    psirz_obmp = psirz[Z0_ind, R0_ind:]
    psipn_obmp = (psirz_obmp - simag) / (sibry - simag)

    sepInd = np.argmin(abs(psipn_obmp - 1.))
    psipn_obmp = psipn_obmp[:sepInd + 1]
    R_obmp = list(R_obmp[:sepInd + 1])

    R = interp(psipn_obmp, R_obmp, psipn)
    if 1 == 0:
        plt.plot(psipn_obmp, R_obmp, label = 'before')
        plt.plot(psipn, R, label = 'after')
        plt.xlabel('psipn')
        plt.ylabel('R')
        plt.legend(loc = 2)
        plt.show()

    EFITdict['R'] = R    # major radius (m) on psipn grid

    #jtor = rmaxis * Pprime + FFprime / rmaxis
    jtor = R * Pprime + FFprime / R
    EFITdict['jtor'] = jtor    # toroidal current density on psipn grid

    #psirz_spl = interpolate.RectBivariateSpline(Zgrid, Rgrid, psirz)

    Bp_Z_grid = np.empty(np.shape(psirz))
    for i in range(nh):
        Bp_Z_grid_this = first_derivative(psirz[i,:].flatten(), Rgrid) / Rgrid
        Bp_Z_grid[i,:] = Bp_Z_grid_this.copy()

    Bp_R_grid = np.empty(np.shape(psirz))
    for i in range(nw):
        Bp_R_grid_this = - first_derivative(psirz[:,i].flatten(), Zgrid) / Rgrid[i]
        Bp_R_grid[:,i] = Bp_R_grid_this.copy()

    #Bp_R_spl = interpolate.RectBivariateSpline(Zgrid, Rgrid, Bp_R_grid)
    #Bp_Z_spl = interpolate.RectBivariateSpline(Zgrid, Rgrid, Bp_Z_grid)

    Bp_tot_grid = np.sqrt(Bp_R_grid ** 2 + Bp_Z_grid ** 2)
    Bp_obmp = Bp_tot_grid[Z0_ind, R0_ind : R0_ind + sepInd + 1]

    Bpol = interp(psipn_obmp, Bp_obmp, psipn)
    EFITdict['Bpol'] = Bpol    # B_pol on psipn grid

    F_spl = interpolate.UnivariateSpline(psipn, Fpol)
    Btor = F_spl(psipn) / R
    EFITdict['Btor'] = abs(Btor)    #B_tor on psipn grid

    return EFITdict

def magneticShear(EFITdict, show_plots = False):

    rhotn = EFITdict['rhotn']
    q = EFITdict['qpsi']

    #uni_rhot = np.linspace(rhotn[0], rhotn[-1], len(rhotn) * 10)
    uni_rhot = np.linspace(rhotn[0], rhotn[-1], len(rhotn))
    q_unirhot = interp(rhotn, q, uni_rhot)
    shat_unirhot = uni_rhot / q_unirhot * first_derivative(q_unirhot, uni_rhot)
    shat = interp(uni_rhot, shat_unirhot, rhotn)

    R_unirhot = interp(rhotn, EFITdict['R'], uni_rhot)
    Ls_unirhot = q_unirhot * R_unirhot / shat_unirhot
    Ls = interp(uni_rhot, Ls_unirhot, rhotn)

    if show_plots:
        plt.plot(uni_rhot, shat_unirhot)
        plt.ylabel('shat')
        plt.xlabel('rhot')
        plt.axis([0.8, 1., 0., 10.])
        plt.show()
        plt.plot(uni_rhot, Ls_unirhot)
        plt.ylabel('Ls')
        plt.xlabel('rhot')
        plt.axis([0.8, 1., 0., 2.])
        plt.show()

    return uni_rhot, shat_unirhot, Ls_unirhot



from scipy import interpolate
import numpy as np
import matplotlib.pyplot as plt

def interp(xin,yin,xnew):
    """
    xin: x variable input
    yin: y variable input
    xnew: new x grid on which to interpolate
    yout: new y interpolated on xnew
    """

    #splrep returns a knots and coefficients for cubic spline
    rho_tck = interpolate.splrep(xin,yin)
    #Use these knots and coefficients to get new y
    yout = interpolate.splev(xnew,rho_tck,der=0)

    return yout

def interp_lin(xin,yin,xnew):
    """
    xin: x variable input
    yin: y variable input
    xnew: new x grid on which to interpolate
    yout: new y interpolated on xnew
    """

    if xnew[0] < xin[0]:
       low_index = np.argmin(abs(xnew-xin[0]))
       if xnew[low_index] < xin[0]:
           low_index += 1
    else:
       low_index = 0
    if xnew[-1] > xin[-1]:
       high_index = np.argmin(abs(xnew-xin[-1]))
       if xnew[high_index] > xin[0]:
           high_index -= 1
    else:
        high_index = -1

    ynew = interpolate.interp1d(xin,yin)
    yout = np.zeros(len(xnew))
    yout[low_index:high_index] = ynew(xnew[low_index:high_index])

    return yout


def full_interp(func_xin,xin,xconv,yconv,yout):
    """
    Takes function func_xin on grid xin and outputs the function on yout grid
    func_xin: function to interpolate
    xin: grid corresponding to func_xin
    xconv: xgrid for conversion
    yconv: ygrid for conversion
    yout: output grid
    """

    #If necessary truncate func_xin onto correct range
    if xin[0] < xconv[0]:
        low_index = np.argmin(abs(xconv-xin[0]))
    else:
        low_index = 0
    if xin[-1] > xconv[-1]:
        high_index = np.argmin(abs(xconv-xin[-1]))
    else:
        high_index = -1

    if high_index == -1:
        func_xin = func_xin[low_index:]
    else:
        func_xin = func_xin[low_index:high_index]

    func_xconv = interp(xin,func_xin,xconv)
    func_yout = interp(yconv,func_xconv,yout)

    return func_yout

def full_interp_lin(func_xin,xin,xconv,yconv,yout):
    """
    Takes function func_xin on grid xin and outputs the function on yout grid
    func_xin: function to interpolate
    xin: grid corresponding to func_xin
    xconv: xgrid for conversion
    yconv: ygrid for conversion
    yout: output grid
    """

    func_xconv = interp_lin(xin,func_xin,xconv)
    func_yout = interp_lin(yconv,func_xconv,yout)

    return func_yout





from SLiM_NN.Dispersion_NN import Dispersion_NN

path='./SLiM_NN/Trained_model/'

[nu, zeff, eta, shat,  beta,  ky,   mu, xstar]=\
[1., 1.5,  1.5, 0.001, 0.001, 0.03, 0., 10.  ]

Dispersion_NN_obj=Dispersion_NN(path)
w=Dispersion_NN_obj.Dispersion_omega(nu,zeff,eta,shat,beta,ky,mu,xstar)

print('w='+str(w))


import matplotlib.pyplot as plt
import sys
import os
from glob import glob
import numpy as np
sys.path.insert(1, './Tools')

from .Tools.read_profiles import read_profile_file
from .Tools.read_profiles import read_geom_file
from .Tools.read_EFIT_file import get_geom_pars
from .Tools.read_pfile import read_pfile_to_dict
from .Tools.read_pfile import psi_rhot
from .Tools.write_iterdb import output_iterdb
from .Tools.write_pfile import write_pfile
from .Tools.write_gfile import mod_q_gfile
from .Tools.interp import interp


def dir_scaner(base_dir,level=1):   
    base_dir=base_dir+'/'
    for i in range(level):
        base_dir=base_dir+"*/"
    directory=glob(base_dir, recursive = True)
    
    dir_list=directory
    return dir_list


def get_g_and_p_file(dir_name):
    file_list=os.listdir(dir_name)
    gfile=''
    pfile=''
    for file in file_list:
        if file[:2]=='g1' or file[:2]=='g2':
            gfile=dir_name+file
        elif file[:2]=='p1' or file[:2]=='g2':
            pfile=dir_name+file

    return gfile,pfile

def get_g_and_p_file_list(dir_name):
    file_list=os.listdir(dir_name)

    g_info={}
    p_info={}
    g_info['shot_num']=[]
    g_info['time']=[]
    g_info['filename']=[]
    p_info['shot_num']=[]
    p_info['time']=[]
    p_info['filename']=[]
    for file in file_list:
        if file[:2]=='g1' or file[:2]=='g2':
            g_info['filename'].append(file)
            str_tmp=file.split('.')
            try:
                g_info['shot_num'].append(int(str_tmp[0][1:]))
            except:
                g_info['shot_num'].append(0)

            try:
                g_info['time'].append(float(str_tmp[1]))
            except:
                g_info['time'].append(0.)

        elif file[:2]=='p1' or file[:2]=='p2':
            p_info['filename'].append(file)
            str_tmp=file.split('.')
            try:
                p_info['shot_num'].append(int(str_tmp[0][1:]))
            except:
                p_info['shot_num'].append(0)

            try:
                p_info['time'].append(float(str_tmp[1]))
            except:
                p_info['time'].append(0.)

    #now get the matching shot number and time

    pg_files={}
    pg_files['shot_num']=[]
    pg_files['time']=[]
    pg_files['pfile']=[]
    pg_files['gfile']=[]
    for i in range(len(g_info['filename'])):
        for j in range(len(p_info['filename'])):
            if g_info['shot_num'][i]==p_info['shot_num'][j] and \
                g_info['time'][i]==p_info['time'][j]:
                pg_files['shot_num'].append(g_info['shot_num'][i])
                pg_files['time'].append(g_info['time'][i])
                pg_files['pfile'].append(p_info['filename'][j])
                pg_files['gfile'].append(g_info['filename'][i])

    return pg_files

class file_IO_obj:
    profile_type=('pfile','ITERDB')
    geomfile_type=('gfile','GENE_tracor')
    def __init__(self,profile_type,profile_name,\
                    geomfile_type,geomfile_name,\
                    suffix='.dat'):
        self.profile_name=profile_name
        self.geomfile_name=geomfile_name
        self.suffix=suffix

        if profile_type not in file_IO_obj.profile_type:
            raise ValueError(f'{profile_type} is not a valid profile type, need to be pfile or ITERDB')
        else: 
            self.profile_type  = profile_type

        if geomfile_type not in file_IO_obj.geomfile_type:
            raise ValueError(f'{geomfile_type} is not a valid geomfile_type, need to be gfile or GENE_tracor')
        else: 
            self.geomfile_type  = geomfile_type

        #*************Loading the data******************************************
        if self.profile_type=="ITERDB":
            rhot0, rhop0, te0, ti0, ne0, ni0, nz0, vrot0 \
                = read_profile_file(self.profile_type,\
                                    self.profile_name,\
                                    self.geomfile_name,\
                                    self.suffix)
        elif self.profile_type=="pfile":
            rhot0, rhop0, te0, ti0, ne0, ni0, nz0, vrot0 \
                = read_profile_file(self.profile_type,\
                                    self.profile_name,\
                                    self.geomfile_name,\
                                    self.suffix)

        if self.geomfile_type=="gfile": 
            xgrid, q, R_ref= read_geom_file(self.geomfile_type,\
                                            self.geomfile_name,\
                                            self.suffix)
        elif self.geomfile_type=="GENE_tracor":
            xgrid, q, Lref, R_ref, Bref, x0_from_para\
                 = read_geom_file(self.geomfile_type,\
                                    self.geomfile_name,\
                                    self.suffix)


        uni_rhot = np.linspace(min(rhot0),max(rhot0),2000)

        self.x    = uni_rhot
        self.rhot = uni_rhot
        self.rhop = np.interp(uni_rhot,rhot0,rhop0)
        self.  te = np.interp(uni_rhot,rhot0,te0)
        self.  ti = np.interp(uni_rhot,rhot0,ti0)
        self.  ne = np.interp(uni_rhot,rhot0,ne0)
        self.  ni = np.interp(uni_rhot,rhot0,ni0)
        self.  nz = np.interp(uni_rhot,rhot0,nz0)
        self.vrot = np.interp(uni_rhot,rhot0,vrot0)
        

        self.xgrid=uni_rhot
        self.q=    np.interp(uni_rhot,xgrid,q)
        self.R_ref=R_ref


    def output_profile(self,profile_type,profile_name,shot_num=999999,time_str='1000'):
        if profile_type not in file_IO_obj.profile_type:
            raise ValueError(f'{profile_type} is not a valid profile type, need to be pfile or ITERDB')
        else: 
            pass
        if profile_type=="ITERDB":
            self.output_ITERDB(profile_name,shot_num,time_str)
        elif profile_type=="pfile":
            self.output_pfile(profile_name)

    def output_geofile(self,geomfile_type,geomfile_name):
        if geomfile_type not in file_IO_obj.geomfile_type:
            raise ValueError(f'{geomfile_type} is not a valid profile type, need to be pfile or ITERDB')
        else: 
            pass
        if geomfile_type=="gfile":
            self.output_gfile(geomfile_name)
        elif geomfile_type=="GENE_tracor":
            print('Stay tunned for the GENE_tracor output capability. ')


    def output_pfile(self,output_profile_name):
        pfile_dic=read_pfile_to_dict(self.profile_name)
        
        psi,rhot=psi_rhot(self.geomfile_name)

        x = self.x
        ne  =self.ne*1.E-20
        te  =self.te*1.E-3
        vrot=self.vrot

        needed_quant={'ne':ne,
                    'te':te,
                    'vtor':vrot}
        needed_quant_key=list(needed_quant.keys())

        #obj_* is the profile generated inside of the obj, i.e. output profile
        #dic_* is the profile in the dictionary readed from p file, i.e. nominal profile

        keys_name_in_dic={}
        for i in range(len(needed_quant)):
            for key in pfile_dic.keys():
                if needed_quant_key[i] in key:
                    f_tmp=needed_quant[needed_quant_key[i]]
                    df_tmp=np.gradient(f_tmp,self.x)
                    keys_name_in_dic[needed_quant_key[i]]={'name':key,\
                                                        'dic_psi':pfile_dic[key]['psi'],\
                                                        'obj_x':self.x,\
                                                        'dic_f':pfile_dic[key]['f'],\
                                                        'obj_f':f_tmp,\
                                                        'dic_df':pfile_dic[key]['df'],\
                                                        'obj_df':df_tmp
                                                        }
        for key in needed_quant_key:
            p_x_psi=keys_name_in_dic[key]['dic_psi']
            output_x_rho= keys_name_in_dic[key]['obj_x']
            output_f_rho= keys_name_in_dic[key]['obj_f']
            output_df_rho=keys_name_in_dic[key]['obj_df']
            
            p_x_rhot=np.interp(p_x_psi,psi,rhot)

            new_dic_f =np.interp(p_x_rhot,output_x_rho,output_f_rho)
            new_dic_df =np.interp(p_x_rhot,output_x_rho,output_df_rho)

            dic_key=keys_name_in_dic[key]['name']
            print(key)
            print(dic_key)
            
            pfile_dic[dic_key]['f']=new_dic_f
            pfile_dic[dic_key]['df']=new_dic_df

        write_pfile(output_profile_name,pfile_dic)
        
    
    def output_gfile(self,output_geofile_name):
        mod_q_gfile(self.geomfile_name,output_geofile_name,self.x,self.q)
        
    

    def output_ITERDB(self,out_iterdb_name,shot_num=999999,time_str='1000'):
        output_iterdb(self.rhot,self.rhop,self.ne*1.E-19,\
                        self.te*1.E-3,self.ni*1.E-19,\
                        self.ti*1.E-3,out_iterdb_name,\
                        shot_num,time_str,\
                        vrot=self.vrot,nimp=self.nz*1.E-19)


    def find_pedestal(self,x,p):

        if np.min(x)<0.65:
            num=int(len(x)*0.7)
            x=(x[num:]).copy()
            p=(p[num:]).copy()
        dp=np.gradient(p,x)  #Second order of pressure
        ddp   = np.gradient(dp,x)  #Second order of pressure
        midped = x[np.argmin(dp)]
        topped = x[np.argmin(ddp)]
        return topped,midped


    def modify_profile(self,\
                        q_scale=1.,q_shift=0.,\
                        shat_scale=1.,\
                        ne_scale=1.,te_scale=1.,\
                        ne_shift=0.,te_shift=0.,\
                        Doppler_scale=1.,\
                        show_plot=False):
        #************start of modification***********
        #********************************************

        #**********************************
        #*************modify ne************
        try:
            self.ne_weight
        except:
            ne_top_ped,ne_mid_ped=self.find_pedestal(self.x,self.ne)
            self.ne_mid_ped=ne_mid_ped
            ne_width=ne_mid_ped-ne_top_ped
            self.ne_weight = 0.5+0.5*np.tanh((self.x-ne_top_ped)/ne_width)
        

        index=np.argmin(abs(self.x-self.ne_mid_ped))
        ne_mod=self.ne*(1.+(ne_scale-1.)*self.ne_weight)+(ne_shift)*self.ne[index]
        #*************modify ne************
        #**********************************

        #**********************************
        #*************modify Te************
        try:
            self.te_weight
        except:
            te_top_ped,te_mid_ped=self.find_pedestal(self.x,self.te)
            self.te_mid_ped=te_mid_ped
            te_width=te_mid_ped-te_top_ped
            self.te_weight = 0.5+0.5*np.tanh((self.x-te_top_ped)/te_width)

        index=np.argmin(abs(self.x-self.te_mid_ped))
        te_mod=self.te*(1.+(te_scale-1.)*self.te_weight)+(te_shift)*self.te[index]
        #*************modify Te************
        #**********************************

        mid_ped,top_ped=self.find_pedestal(self.x,self.te)


        width=(mid_ped-top_ped)*10.
        try:
            self.shat_weight
        except:
            self.shat_weight = -1.*np.tanh((self.x-mid_ped)/width)/2
        
        q_mod=self.q*q_scale*(1.+(shat_scale-1.)*self.shat_weight)+q_shift

        vrot_mod=self.vrot*Doppler_scale

        #show_plot=True

        if show_plot==True:
            fig, ax=plt.subplots(nrows=4,ncols=1,sharex=True) 
            ax[0].plot(self.x,self.ne,label='original')
            ax[0].plot(self.x,ne_mod,label='modified')
            ax[0].legend()
            ax[0].set_ylabel(r'$n_e(10^19/m^3)$',fontsize=10)
            ax[1].plot(self.x,self.te)
            ax[1].plot(self.x,te_mod)
            ax[1].set_ylabel(r'$T_e(keV)$',fontsize=10)
            ax[2].plot(self.x,self.q)
            ax[2].plot(self.x,q_mod)
            ax[2].set_ylabel(r'$q$',fontsize=10)
            ax[3].plot(self.x,self.vrot)
            ax[3].plot(self.x,vrot_mod)
            ax[3].set_ylabel(r'$vrot$',fontsize=10)
            ax[3].set_xlabel(r'$\rho_{tor}$',fontsize=10)
            plt.tight_layout()
            plt.show()

        #*************end of modification************
        #********************************************

        self.x =self.x 
        self.ne=ne_mod      # in 10^19 /m^3
        self.te=te_mod     # in keV 
        self.q=q_mod
        self.vrot=vrot_mod



import numpy as np
import matplotlib.pyplot as plt
import re

def read_iterdb(filename):
    f=open(filename,'r')
    data_in=f.read()
    data_linesplit=data_in.split('\n')

    keep_going=1
    i=0
    while keep_going:
        test=re.search(';-# OF X PTS',data_linesplit[i])
        if test:
            num=data_linesplit[i].split()[0]
            num=float(num)
            num=int(num)
            print("number of points:",num)
            keep_going=(1==2)
        if i == len(data_linesplit):
            keep_going=(1==2)
        i=i+1

    lnum=0
    try_again=1
    prof_out = {}
    rhot_out = {}
    units_out = {}
    while try_again:
        lnum,try_again,quantity,units,rhot,arr=get_next(data_linesplit,lnum,num)
        prof_out[quantity]=arr
        units_out[quantity]=units
        rhot_out[quantity]=rhot
    return rhot_out,prof_out,units_out   
     

def get_next(data_linesplit,lnum,num):
    sec_num_lines = num/6
    if num % 6 != 0:
        sec_num_lines += 1
    keep_going=1
    while keep_going:
        test=re.search('-DEPENDENT VARIABLE LABEL',data_linesplit[lnum])
        #test2=re.search('INDEPENDENT',data_linesplit[lnum])
        if test :
            quantity=data_linesplit[lnum].split()[0]
            units=data_linesplit[lnum].split()[1]
        test=re.search('DATA FOLLOW',data_linesplit[lnum])
        if test:
            keep_going=(1==2)
        lnum=lnum+1
    
    rhot=np.empty(0)
    lnum0 = lnum
    for j in range(lnum0,lnum0+sec_num_lines):
        for k in range(6):
            str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
            if(str_temp):
                temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                rhot=np.append(rhot,temp)
        lnum=lnum+1
    #print rhot
    #print "var=",quantity
    lnum=lnum+1
    
    arr=np.empty(0)
    lnum0 = lnum
    for j in range(lnum0,lnum0+sec_num_lines):
        for k in range(6):
            str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
            if(str_temp):
                temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                arr=np.append(arr,temp)
        #arr_str=data_linesplit[j].split()
        #arr_flt=np.array(arr_str,dtype='float')
        #arr=np.append(arr,arr_flt)
        lnum=lnum+1
    #print 'rhot',rhot
    #print 'arr',arr
    #print len(rhot),len(arr)
    #if quantity=='VROT':
    #    vout=np.empty((len(arr),2),dtype='float')
    #    vout[:,0]=rhot
    #    vout[:,1]=arr
    #    f=open(quantity+'.dat','w')
    #    f.write('#'+file_name+'\n'+'#rhot '+quantity+'\n')
    #    np.savetxt(f,vout)
    #    f.close()
    
    #plt.plot(rhot,arr,label=quantity+'('+units+')')
    #plt.xlabel(r'$\rho_{tor}$',size=18)
    #plt.legend(loc='lower left')
    #plt.show()
    
    #if gene_plots and quantity=='TI':
    #    plt.plot(rhot,arr/1000.0,label=quantity+'('+units+')')
    #    plt.plot(gprof_i[:,0],gprof_i[:,2],'r.',label='$T_i$ gene')
    #    plt.legend(loc='lower left')
    #    plt.show()

    lnum_out=lnum
    try_again=1
    if len(data_linesplit)-lnum < 10:
        try_again=False
    return lnum_out, try_again,quantity,units,rhot,arr





import tensorflow as tf  
import pandas as pd 
import numpy as np 
import os

#stability, omega 2 net
class Dispersion_NN():
    def __init__(self,download_path='./'):
        if download_path[-1]!='/':
            download_path=download_path+'/'
        
        if os.path.exists(download_path):
            pass
        else:
            

        NN_stability_model = tf.keras.models.load_model(self.NN_stability_file)
        NN_omega_model = tf.keras.models.load_model(self.NN_omega_file)
        
        NN_stability_model.summary()
        NN_omega_model.summary()

        df_stability=pd.read_csv(self.norm_stability_csv_file)
        df_omega=pd.read_csv(self.norm_omega_csv_file)
        
        self.NN_stability_model=NN_stability_model
        self.NN_omega_model=NN_omega_model
        
        self.norm_stability_factor=df_stability
        self.norm_omega_factor=df_omega



    def Dispersion_omega(self,nu,zeff,eta,shat,beta,ky,mu,xstar):
        param=np.array([nu,zeff,eta,shat,beta,ky,mu/xstar])
        param_omega_norm=np.zeros(7,dtype=float)
        param_stability_norm=np.zeros(7,dtype=float)
                  #nu, zeff, eta, shat, beta, ky, mu/xstar  gamma
        log_scale=[1,  0,    0,   1,    1,    0,  0       , 0    ]

        for i in range(7):
            if log_scale[i]==1:
                param_temp=np.log(param[i])
            else:
                param_temp=param[i]
            param_stability_norm[i]=(param_temp-self.norm_stability_factor['offset'][i])*self.norm_stability_factor['factor'][i]
            param_omega_norm[i]    =(param_temp-self.norm_omega_factor['offset'][i])    *self.norm_omega_factor['factor'][i]
            

        [[stability]] = self.NN_stability_model.predict(np.array([param_stability_norm]),verbose = 0)
        

        if stability>0.2:
            [[omega]] = self.NN_omega_model.predict(np.array([param_omega_norm]),verbose = 0)
            gamma=1.
        else:
            gamma=0.
            omega=0.

        #print(self.norm_omega_factor)

        omega=omega/(self.norm_omega_factor['factor'][7])+(self.norm_omega_factor['offset'][7])
        w=omega+gamma*1j
        
        return w


#gamma, omega 3 net
class Dispersion_NN_beta3():
    def __init__(self,NN_path):
        if NN_path[-1]!='/':
            NN_path=NN_path+'/'

        self.NN_stability_file=NN_path+'SLiM_NN_stability.h5'
        self.NN_omega_file=NN_path+'SLiM_NN_omega.h5'
        self.NN_gamma_file=NN_path+'SLiM_NN_gamma.h5'

        self.norm_stability_csv_file=NN_path+'NN_stability_norm_factor.csv'
        self.norm_omega_csv_file=NN_path+'NN_omega_norm_factor.csv'
        self.norm_gamma_csv_file=NN_path+'NN_gamma_norm_factor.csv'

        NN_stability_model = tf.keras.models.load_model(self.NN_stability_file)
        NN_omega_model = tf.keras.models.load_model(self.NN_omega_file)
        NN_gamma_model = tf.keras.models.load_model(self.NN_gamma_file)
        
        
        NN_stability_model.summary()
        NN_omega_model.summary()
        NN_gamma_model.summary()

        df_stability=pd.read_csv(self.norm_stability_csv_file)
        df_omega=pd.read_csv(self.norm_omega_csv_file)
        df_gamma=pd.read_csv(self.norm_gamma_csv_file)

        self.NN_stability_model=NN_stability_model
        self.NN_omega_model=NN_omega_model
        self.NN_gamma_model=NN_gamma_model

        self.norm_stability_factor=df_stability
        self.norm_omega_factor=df_omega
        self.norm_gamma_factor=df_gamma



    def Dispersion_omega(self,nu,zeff,eta,shat,beta,ky,mu,xstar):
        param=np.array([nu,zeff,eta,shat,beta,ky,mu/xstar])
        param_omega_norm=np.zeros(7,dtype=float)
        param_gamma_norm=np.zeros(7,dtype=float)
        param_stability_norm=np.zeros(7,dtype=float)
                  #nu, zeff, eta, shat, beta, ky, mu/xstar  gamma
        log_scale=[1,  0,    0,   1,    1,    0,  0       , 0    ]

        for i in range(7):
            if log_scale[i]==1:
                param_temp=np.log(param[i])
            else:
                param_temp=param[i]
            param_stability_norm[i]=(param_temp-self.norm_stability_factor['offset'][i])*self.norm_stability_factor['factor'][i]
            param_omega_norm[i]    =(param_temp-self.norm_omega_factor['offset'][i])    *self.norm_omega_factor['factor'][i]
            param_gamma_norm[i]    =(param_temp-self.norm_gamma_factor['offset'][i])    *self.norm_gamma_factor['factor'][i]

        [[stability]] = self.NN_stability_model.predict(np.array([param_stability_norm]),verbose = 0)
        

        if stability>0.5:
            gamma = self.NN_gamma_model.predict(np.array([param_gamma_norm]),verbose = 0)
            omega = self.NN_omega_model.predict(np.array([param_omega_norm]),verbose = 0)
            print('\ngamma='+str(gamma))
            gamma=1.
        else:
            gamma=0.
            omega=0.

        #print(self.norm_omega_factor)

        omega=omega/(self.norm_omega_factor['factor'][7])+(self.norm_omega_factor['offset'][7])
        gamma=gamma/(self.norm_gamma_factor['factor'][7])+(self.norm_gamma_factor['offset'][7])
        w=omega+gamma*1j
        
        return w


        
#gamma, omega 2 net
class Dispersion_NN_beta():
    def __init__(self,NN_omega_file,NN_gamma_file,\
                norm_omega_csv_file,norm_gamma_csv_file):
        NN_omega_model = tf.keras.models.load_model(NN_omega_file)
        NN_gamma_model = tf.keras.models.load_model(NN_gamma_file)
        
        NN_omega_model.summary()
        NN_gamma_model.summary()

        df_omega=pd.read_csv(norm_omega_csv_file)
        df_gamma=pd.read_csv(norm_gamma_csv_file)

        self.NN_omega_file=NN_omega_file
        self.NN_gamma_file=NN_gamma_file
        self.norm_omega_csv_file=norm_omega_csv_file
        self.norm_gamma_csv_file=norm_gamma_csv_file

        self.NN_omega_model=NN_omega_model
        self.NN_gamma_model=NN_gamma_model
        self.norm_omega_factor=df_omega['factor']
        self.norm_gamma_factor=df_gamma['factor']

    def Dispersion_omega(self,nu,zeff,eta,shat,beta,ky,mu,xstar):
        param=[nu,zeff,eta,shat,beta,ky,mu/xstar]
        param_omega_norm=np.zeros(7,dtype=float)
        param_gamma_norm=np.zeros(7,dtype=float)

        for i in range(7):
            param_omega_norm[i]=param[i]*self.norm_omega_factor[i]
            param_gamma_norm[i]=param[i]*self.norm_gamma_factor[i]

        omega = self.NN_omega_model.predict(np.array([param_omega_norm]),verbose = 0)
        gamma = self.NN_gamma_model.predict(np.array([param_gamma_norm]),verbose = 0)

        [[w]]=omega+gamma*1j

        return w



#gamma, omega 1 net
class Dispersion_NN_beta_2():
    def __init__(self,NN_file,norm_csv_file):
        NN_model = tf.keras.models.load_model(NN_file)
        
        NN_model.summary()

        df=pd.read_csv(norm_csv_file)


        self.NN_file=NN_file
        self.norm_csv_file=norm_csv_file
        self.NN_model=NN_model
        self.norm_factor=df


    def Dispersion_omega(self,nu,zeff,eta,shat,beta,ky,mu,xstar):
        param=[nu,zeff,eta,shat,beta,ky,mu/xstar]
        param_norm=np.zeros(7,dtype=float)

                  #nu, zeff, eta, shat, beta, ky, mu/xstar  gamma omega 
        log_scale=[1,  0,    0,   1,    1,    0,  0       , 0    , 0   ]

        for i in range(7):
            if log_scale[i]==1:
                param_temp=np.log(param[i])
            else:
                param_temp=param[i]
            param_norm[i]=(param_temp-self.norm_factor['offset'][i])*self.norm_factor['factor'][i]
            

        [[omega,gamma]] = self.NN_model.predict(np.array([param_norm]),verbose = 0)
        

        omega=omega/(self.norm_factor['factor'][7])+(self.norm_factor['offset'][7])
        gamma=gamma/(self.norm_factor['factor'][8])+(self.norm_factor['offset'][8])

        w=omega+gamma*1j

        return w



#Created by Max T. Curie  03/29/2022
#Last edited by Max Curie 03/29/2022

#the input will be dictionary
def write_pfile(filename,p_quant_dict):
    file=open(filename,"w")
    
    for key in p_quant_dict.keys():
        header=p_quant_dict[key]['header']
        psi=p_quant_dict[key]['psi']
        f=p_quant_dict[key]['f']
        df=p_quant_dict[key]['df']

        write_quant(file,header,psi,f,df)
    
def write_quant(file,header,psi,f,df):
    file.write(header+'\n')
    for psi_i,f_i,df_i in zip(psi,f,df):
        file.write(' %.6f   %.6f   %.6f\n'%(psi_i,f_i,df_i))



#!/usr/bin/env python

import numpy as np
import re
import scipy as sc
import matplotlib.pyplot as plt
from scipy import interpolate

def iterdb_header(quant_str,units,quant_len,shot_num):
    #header=' '+shot_num+'              ;-SHOT #- F(X) DATA \n'
    header=' 99999  xyz 2              ;-SHOT #- F(X) DATA \n'
    header=header+'                              ;-SHOT DATE-  UFILES ASCII FILE SYSTEM\n'
    header=header+'   0                          ;-NUMBER OF ASSOCIATED SCALAR QUANTITIES\n'
    header=header+' RHOTOR              -        ;-INDEPENDENT VARIABLE LABEL: X-\n'
    header=header+' TIME                SECONDS  ;-INDEPENDENT VARIABLE LABEL: Y-\n'
    spaces = '                '
    if(len(quant_str)==3):
        spaces += ' '
    elif(len(quant_str)==2):
        spaces += '  '
    header=header+' '+quant_str+spaces+units+'           ;-DEPENDENT VARIABLE LABEL\n'
    header=header+' 3                            ;-PROC CODE- 0:RAW 1:AVG 2:SM. 3:AVG+SM\n'
    header=header+'      '+str(quant_len)+'                   ;-# OF X PTS- \n'
    header=header+'      1                   ;-# OF Y PTS-  X,Y,F(X,Y) DATA FOLLOW:\n'
    #header+=' '
    return header 

def iterdb_write_quant(fileid,quant_arr):
    fileid.write(' ')
    for i in range(len(quant_arr)):
        out_str='%-12e' % quant_arr[i]
        #if quant_arr[i] > 0.0:
        if quant_arr[i] >= 0.0:
            out_str=' '+out_str
        fileid.write(out_str)
        if (i+1)%6==0 and i != len(quant_arr)-1:
            fileid.write('\n')
            fileid.write(' ')
    fileid.write('\n')

def output_iterdb(rhot,rhop,ne,te,ni,ti,file_base,shot_num,time_string,vrot=[-999],nimp=[-999]):
    transition=';----END-OF-DATA-----------------COMMENTS:-----------\n'
    transition=transition+'********************************************************************************\n'
    transition=transition+'********************************************************************************\n'
    idbf=open(file_base+'.iterdb','w')
    header=';Created with script write_iterdb.py for GENE input\n' 
    idbf.write(header)
    header=';----END-OF-ORIGINAL-HEADER------COMMENTS:-----------\n'
    idbf.write(header)

    header=iterdb_header('TE','eV',len(rhot),shot_num)
    idbf.write(header)
    iterdb_write_quant(idbf,rhot)
    idbf.write('  '+time_string+'\n')      
    iterdb_write_quant(idbf,1000.0*te)
    idbf.write(transition)

    header=iterdb_header('TI','eV',len(rhot),shot_num)
    idbf.write(header)
    iterdb_write_quant(idbf,rhot)
    idbf.write('  '+time_string+'\n')      
    iterdb_write_quant(idbf,1000.0*ti)
    idbf.write(transition)

    header=iterdb_header('NE','m^-3',len(rhot),shot_num)
    idbf.write(header)
    iterdb_write_quant(idbf,rhot)
    idbf.write('  '+time_string+'\n')      
    iterdb_write_quant(idbf,1.0e19*ne)
    idbf.write(transition)

    header=iterdb_header('NM1','m^-3',len(rhot),shot_num)
    idbf.write(header)
    iterdb_write_quant(idbf,rhot)
    idbf.write('  '+time_string+'\n')      
    iterdb_write_quant(idbf,1.0e19*ni)
    idbf.write(transition)

    #Include impurity density
    if nimp[0] != -999:
        header=iterdb_header('NM2','m^-3',len(rhot),shot_num)
        idbf.write(header)
        iterdb_write_quant(idbf,rhot)
        idbf.write('  '+time_string+'\n')      
        iterdb_write_quant(idbf,1.0e19*nimp)
        idbf.write(transition)

    if vrot[0] != -999:
        header=iterdb_header('VROT','rad/s',len(rhot),shot_num)
        idbf.write(header)
        iterdb_write_quant(idbf,rhot)
        idbf.write('  '+time_string+'\n')      
        iterdb_write_quant(idbf,vrot)
        idbf.write(transition)

    idbf.close()





import matplotlib.pyplot as plt
import numpy as np
import re

def read_iterdb_file(iterdb_filename):

    iterdb_file=open(iterdb_filename,'r')
    with_impurity = False

    data_in=iterdb_file.read()
    data_linesplit=data_in.split('\n')

    keep_going=1
    i=0
    while keep_going:
        test=re.search(';-# OF X PTS',data_linesplit[i])
        if test:
            num=data_linesplit[i].split()[0]
            num=float(num)
            num=int(num)
            print( "number of points in iterdb file:",num)
            keep_going=(1==2)
        if i == len(data_linesplit):
            keep_going=(1==2)
        i=i+1

    lnum=0
    while len(data_linesplit)-lnum > 10:
        sec_num_lines = num/6
        if num % 6 != 0:
            sec_num_lines += 1
        keep_going=1
        while keep_going and len(data_linesplit) > lnum:
            #print "len(data_linesplit)",len(data_linesplit)
            #print "lnum",lnum
            test=re.search('-DEPENDENT VARIABLE LABEL',data_linesplit[lnum])
            #test2=re.search('INDEPENDENT',data_linesplit[lnum])
            if test :
                quantity=data_linesplit[lnum].split()[0]
                units=data_linesplit[lnum].split()[1]
            test2=re.search('DATA FOLLOW',data_linesplit[lnum])
            if test2:
                keep_going=(1==2)
            lnum=lnum+1

        if quantity=='TE':
           print( "Reading :",quantity)
           rhot_te=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                       temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                       rhot_te=np.append(rhot_te,temp)
               lnum=lnum+1
       
           lnum=lnum+1

           te=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                      temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                      te=np.append(te,temp)
               lnum=lnum+1

        if quantity=='TI':
           print( "Reading :",quantity)
           rhot_ti=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                       temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                       rhot_ti=np.append(rhot_ti,temp)
               lnum=lnum+1

           lnum=lnum+1

           ti=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                      temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                      ti=np.append(ti,temp)
               lnum=lnum+1
#        else:
#           lnum = lnum + 2*sec_num_lines + 1

        if quantity=='NE':
           print( "Reading :",quantity)
           rhot_ne=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                       temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                       rhot_ne=np.append(rhot_ne,temp)
               lnum=lnum+1

           lnum=lnum+1

           ne=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                      temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                      ne=np.append(ne,temp)
               lnum=lnum+1

        if quantity=='NM1':
           print( "Reading :",quantity)
           rhot_ni=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                       temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                       rhot_ni=np.append(rhot_ni,temp)
               lnum=lnum+1

           lnum=lnum+1

           ni=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                      temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                      ni=np.append(ni,temp)
               lnum=lnum+1

        if quantity=='NM2':
           with_impurity = True
           print( "Reading :",quantity)
           rhot_nb=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                       temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                       rhot_nb=np.append(rhot_nb,temp)
               lnum=lnum+1

           lnum=lnum+1

           nb=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                      temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                      nb=np.append(nb,temp)
               lnum=lnum+1

        if quantity=='VROT':
           print( "Reading :",quantity)
           rhot_vrot=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                       temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                       rhot_vrot=np.append(rhot_vrot,temp)
               lnum=lnum+1

           lnum=lnum+1

           vrot=np.empty(0)
           lnum0 = lnum
           for j in range(int(lnum0),int(lnum0+sec_num_lines)):
               for k in range(6):
                   str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
                   if(re.search('e',str_temp)):
                      temp=np.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                      vrot=np.append(vrot,temp)
               lnum=lnum+1
    #plt.plot(rhot_te)
    #plt.plot(rhot_ti)
    #plt.plot(rhot_ne)
    #plt.plot(rhot_ni)
    #plt.plot(rhot_nb)
    #plt.plot(rhot_vrot)
    #plt.show()
    if not with_impurity:
       nb = [False]
    return rhot_te, te, ti, ne, ni, nb, vrot





import numpy as np
import matplotlib.pyplot as plt
import sys
import math

from .interp import *
from .finite_differences_x import *
from .read_EFIT import read_EFIT

#Created by Max T. Curie  11/02/2020
#Last edited by Max Curie 03/29/2020
#Supported by scripts in IFS

def read_pfile(p_file_name):

    f=open(p_file_name,'r')
    data = f.read()
    f.close()

    sdata = data.split('\n')
    
    nr = int(sdata[0].split()[0]) 

    print("p-file resolution: nr = "+str(nr) )

    if (nr+1)*int(len(sdata)/(nr+1))!=sdata:
        print('data need to be managed manually, please look into the code output carefully. ')

    name_list=[]
    
    #for i in range(int(len(sdata)/(nr+1))):   #scan all of the quantitites in the p file
        #print(sdata[i*nr+i].split()[2])
        #name_list.append(sdata[i*nr+i].split()[2])

    need_list=['ne','ni','nz','te','ti','er','vtor']   #List of quantities that need for to be read
    need_list_number=[-1]*len(need_list)          #Number of the namelist that matches with the list of quantities that need for to be read
    
    print('All the quantities listed in the p file: ')
    #print(name_list)
    for i in range(len(need_list)):
        for j in range(len(sdata)):
            if len(sdata[j].split()) > 3:
                if str(need_list[i]) in str(sdata[j].split()[2]):
                    need_list_number[i]=j

    case=0

    print('list of quantities name needed')
    print(need_list)
    print('number coorepsonds to the name list, -1 means missing')
    print(need_list_number)
    #if len(need_list) > len(need_list_number): 
    if -1 in need_list_number:
        print('Error, missing needed quantities in p file, check line 31 need_list in read_qfile.py')
        if need_list_number[4]==-1:
            need_list_number[4]=0
            temp0=input("The array er is missing, force the array to zero, countiune: 1:Yes, 2.No         ")
            if temp0==2:
                sys.exit()
            elif temp0==1:
                case=1    #er missing 
                
        elif need_list_number[5]==-1:
            temp0=input("The array vtor is missing, force the array to zero, countiune: 1:Yes, 2.No        ")
            need_list_number[5]=0
            if temp0==2:
                sys.exit()
            elif temp0==1 and case==1:
                print("Both array vtor and er are missing, cannot calculate doppler shift")
                case=3   #both 
            elif temp0==1 and case==0:
                case=2   #vtor missing
                

        
        #for j in range(len(need_list)):
        #    if need_list_number need_list[j]
        #        print('Error, missing'+str(need_list[j]))
    print(str(name_list))
    print(str(need_list))
    print(str(need_list_number))

    psi=[]
    f=[]
    df=[]
    
    #print(len(need_list_number))
    for i in range(len(need_list_number)):
        n_temp=int(need_list_number[i])   #take the i_th data set
        if n_temp==-1:
            psi_tmp=np.linspace(0,1,nr)
            for j in range(nr):
                psi.append(psi_tmp[j])
                f.append(0.)
                df.append(0.)
        else:
            for j in range(nr):
                temp = sdata[n_temp+j+1].split()
                #print(temp)
                psi.append(float(temp[0]))
                f.append(float(temp[1]))
                df.append(float(temp[2]))
            
        #plt.clf()
        #plt.plot(psi,f)
        #plt.show()

    # ne is electron density profile, dne is gradient of ne
    # psipne is the grid of psi_pol on which ne&dne above is recorded
    temp_i=0
    psipne = psi[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    psipni = psi[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    psipnz = psi[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    psipte = psi[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    psipti = psi[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    psiper = psi[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    psipvtor = psi[temp_i*nr:(temp_i+1)*nr]
    
    temp_i=0
    ne = f[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    ni = f[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    nz = f[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    te = f[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    ti = f[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    er = f[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    vtor = f[temp_i*nr:(temp_i+1)*nr]

    temp_i=0
    dne = df[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    dni = df[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    dnz = df[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    dte = df[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    dti = df[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    der = df[temp_i*nr:(temp_i+1)*nr]
    temp_i=temp_i+1
    dvtor = df[temp_i*nr:(temp_i+1)*nr]

    if case==1: #er missing 
        er=np.zeros(len(er))
        der=np.zeros(len(der))
    elif case==2: #vtor missing
        vtor=np.zeros(len(vtor))
        dvtor=np.zeros(len(dvtor))
    elif case==3: #both are missing
        vtor=np.zeros(len(vtor))
        dvtor=np.zeros(len(dvtor))
        er=np.zeros(len(er))
        der=np.zeros(len(der))

    # psipne is the grid of psi_pol on which ne&dne above is recorded
    

    #plt.plot(psipne,'x')
    #plt.plot(psipni,'r.')
    #plt.plot(psipti,'bo')
    #plt.plot(psiper,'g.')
    #plt.plot(psipvtor,'rx')
    #plt.show()

    #psi0 is normalized psi_pol in [0,1] with 1000 points
    #quantities with _out are interpolated on psi0 grid 
    #print "length of arrays",len(ne),len(ni),len(te),len(ti)
    #psi0 = np.arange(1000)/999.0
    #psi0 = np.linspace(0.,1.,len(ne),endpoint=False)
    psi0 = np.linspace(0.,1.,400)
    ne_out = interp(psipne,ne,psi0)
    te_out = interp(psipte,te,psi0)
    ni_out = interp(psipni,ni,psi0)
    nz_out = interp(psipni,nz,psi0)
    ti_out = interp(psipti,ti,psi0)
    er_out = interp(psiper,er,psi0)
    vtor_out = interp(psipvtor,vtor,psi0)

    dne_out = interp(psipne,dne,psi0)
    dte_out = interp(psipte,dte,psi0)
    dni_out = interp(psipni,dni,psi0)
    dti_out = interp(psipti,dti,psi0)
    dnz_out = interp(psipni,dnz,psi0)
    der_out = interp(psiper,der,psi0)
    dvtor_out= interp(psipvtor,dvtor,psi0)

    
    if np.sum(nz_out)<=0.01:
        Z=6.
        impurity_charge = float(Z)
        nz_out = np.empty(len(ne_out))
        for i in range(len(ne_out)):
            nz_out[i] = (ne_out[i]-ni_out[i])/impurity_charge
    else:
        pass
    
    print(vtor_out)
    print('!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!')
    return psi0, ne_out, te_out, ni_out, ti_out, nz_out, er_out, vtor_out


def read_pfile_to_dict(p_file_name):
    f=open(p_file_name,'r')
    data = f.read()
    f.close()

    sdata = data.split('\n')
    nr = int(sdata[0].split()[0]) 

    print("p-file resolution: nr = "+str(nr) )
    
    p_quant_dict={}

    
    i=0

    while i<len(sdata):
        if i==0:
            psi=[]
            f=[]
            df=[]
            header_index=0
        elif i==(len(sdata)-1):
            name=sdata[header_index].split()[2]
            p_quant_dict[name]={
                            'header_index':header_index,
                            'header':sdata[header_index],
                            'psi':psi,
                            'f':f,
                            'df':df
                            }
            psi=[]
            f=[]
            df=[]
            header_index=i
        elif len(sdata[i].split())>3:
            #write the last data set into the dict
            name=sdata[header_index].split()[2]
            p_quant_dict[name]={
                            'header_index':header_index,
                            'header':sdata[header_index],
                            'psi':psi,
                            'f':f,
                            'df':df
                            }
            psi=[]
            f=[]
            df=[]
            header_index=i    
        else:
            temp = sdata[i].split()
            psi.append(float(temp[0]))
            f.append(float(temp[1]))
            df.append(float(temp[2]))
            
        i=i+1

    return p_quant_dict

def psi_rhot(geomfile_name):
    EFITdict = read_EFIT(geomfile_name)
    psip=EFITdict['psipn']
    rhot=EFITdict['rhotn']

    return psip,rhot

def interp_f_psi(psi,rhot,psi_p,x_rhot,f_rhot):
    x_psi=np.interp(x_rhot,rhot,psi)
    f_psi=np.interp(f_rhot,rhot,psi)
    return x_psi,f_psi

def p_to_iterdb_format(p_file_name,geomfile_name):
    psi0, ne0, te0, ni0, ti0, nz0, er0, vtor_out = read_pfile(p_file_name)
    case=2
    if sum(er0)==0:
        print('Er is empty, using vtor to calculate Shear')
        case=1
    elif sum(vtor_out)==0:
        print('vtor is empty, using Er to calculate Shear')
        case=2
    elif sum(er0)!=0 and sum(er0)!=0:
        print('Neither Er nor vtor is empty, using both to calculate')
        case=3
    elif sum(er0)==0 and sum(er0)==0:
        print('Both Er and vtor are empty, cannot calculate Shear')
        case=4

    EFITdict = read_EFIT(geomfile_name)
    print(str(list(EFITdict.keys())))

    sepInd = np.argmin(abs(EFITdict['psipn'] - 1.))
    print('index at psipn = 1 is '+str(sepInd) )
    Rsep = EFITdict['R'][sepInd]
    print('major R(m) at psipn = 1 is '+str(Rsep))
    print('major R(m) at index = 1 is '+str(EFITdict['R'][0]))
    
    # construct R grid with uniform spacing 
    # uniform spacing because first_derivative requires so
    # find pressure, temperature, density values on uniform R grid

    uni_R = np.linspace(EFITdict['R'][0],Rsep,int(EFITdict['nw']*1.2))
    psip_uniR = interp(EFITdict['R'], EFITdict['psipn'], uni_R)
    rhot_uniR = interp(EFITdict['R'], EFITdict['rhotn'], uni_R)

    rhot0 = interp(EFITdict['psipn'], EFITdict['rhotn'], psi0)
    pi0 = ni0 * ti0
    pi_uniR = interp(rhot0,pi0,rhot_uniR)
    ni_uniR = interp(rhot0,ni0,rhot_uniR)
    ti_uniR = interp(rhot0,ti0,rhot_uniR)
    pe0 = ne0 * te0
    pe_uniR = interp(rhot0,pe0,rhot_uniR)
    ne_uniR = interp(rhot0,ne0,rhot_uniR)
    te_uniR = interp(rhot0,te0,rhot_uniR)
    
    # compute grad P_i / n_i / e, grad P_e / n_e / e 
    gradPioverNe = first_derivative(pi_uniR,uni_R)/ni_uniR 
    gradPeoverNe = first_derivative(pe_uniR,uni_R)/ne_uniR 


    uni_rhot = np.linspace(min(rhot0),max(rhot0),int(len(rhot0)*1.2))
    ti_u = interp(rhot0,ti0,uni_rhot)
    te_u = interp(rhot0,te0,uni_rhot)
    ne_u = interp(rhot0,ne0,uni_rhot)
    ni_u = interp(rhot0,ni0,uni_rhot)
    nz_u = interp(rhot0,nz0,uni_rhot)
    p_u = (ni_u + nz_u) * ti_u + ne_u * te_u


    tprime_i = -first_derivative(ti_u,uni_rhot)/ti_u
    tprime_e = -first_derivative(te_u,uni_rhot)/te_u
    nprime_e = -first_derivative(ne_u,uni_rhot)/ne_u
    nprime_i = -first_derivative(ni_u,uni_rhot)/ni_u
    nprime_z = -first_derivative(nz_u,uni_rhot)/nz_u
    eta_i = tprime_i / nprime_i
    eta_e = tprime_e / nprime_e
    eta_z = tprime_i / nprime_z


    # convert from kV/m to V/m
    Er_Vm = interp(rhot0,er0,uni_rhot)*1E3


    R_u = interp(EFITdict['rhotn'],EFITdict['R'],uni_rhot)
    Bpol_u = interp(EFITdict['rhotn'],EFITdict['Bpol'],uni_rhot)
    vtor_out_u = interp(psi0,vtor_out,uni_rhot)
    
    # add minus sign for consistency
    omega_tor_Er = - Er_Vm / (R_u * Bpol_u)
    #print(R_u[0])
    omega_tor_Vor = vtor_out_u*1000. / (R_u)

    if case==1:
        omega_tor=omega_tor_Vor
    if case==2:
        omega_tor=omega_tor_Er
    if case==3:
        if sum(abs((omega_tor_Er-omega_tor_Vor)/omega_tor_Vor))>0.05*float(len(omega_tor_Vor)):
            print("Too much difference between omega_tor calculated from Er and vtor")
            
            #plt.clf()
            #plt.plot(uni_rhot,omega_tor_Er,label='omega_tor_Er')
            #plt.plot(uni_rhot,omega_tor_Vor,label='omega_tor_Vor')
            #plt.xlabel('rhot')
            #plt.legend()
            #plt.show()
            

            #decide=int(input("omega_tor_Er or omega_tor_Vor, 1. omega_tor_Er, 2. omega_tor_Vor:      "))
            decide=1

            if decide==1:
                omega_tor=omega_tor_Er
            elif decide==2:
                omega_tor=omega_tor_Vor
            else:
                print("please input 1 or 2")
        else:
            omega_tor=omega_tor_Vor
    if case==4:
        print('Both Er and vtor are empty, cannot calculate Shear')
        omega_tor=omega_tor_Er
    # densities are multiplied by 10 here 
    # because output_iterdb() expects density in 10^19 m^-3
    
    psi_u = interp(rhot0,psi0,uni_rhot)
    rhop_u = np.sqrt(np.array(psi_u))

    return uni_rhot, rhop_u, 1000.0*te_u, 1000.0*ti_u, 1.0e19*ne_u*10., 1.0e19*ni_u*10., 1.0e19*nz_u*10., omega_tor






#!/usr/bin/env python
# -*- coding: utf-8 -*-

import os
import re
import sys
import traceback

import numpy as npy
import matplotlib.pyplot as plt
import matplotlib.backends.backend_pdf as pdfh

from scipy.integrate import simps
from scipy.interpolate import interp1d,interp2d
from scipy.interpolate import CubicSpline,RectBivariateSpline

from . import read_iterdb


def bisection(fx,xmin,xmax,root=0.0,Nmax=100,eps=1.0e-16):
    for it in range(Nmax):
        x = (xmax+xmin)/2.0
        if abs(fx(x)-root) < eps: break
        if   (fx(xmax)-root)*(fx(x)-root) < 0.0: xmin = x
        else:                                    xmax = x
    return x

def rk4(f1,f2,czt,slns,stp):
    k01 = f1(slns[0],slns[1])
    k11 = f2(slns[0],slns[1])
    k02 = f1(slns[0]+stp*k01/2.0,slns[1]+stp*k11/2.0)
    k12 = f2(slns[0]+stp*k01/2.0,slns[1]+stp*k11/2.0)
    k03 = f1(slns[0]+stp*k02/2.0,slns[1]+stp*k12/2.0)
    k13 = f2(slns[0]+stp*k02/2.0,slns[1]+stp*k12/2.0)
    k04 = f1(slns[0]+stp*k03,slns[1]+stp*k13)
    k14 = f2(slns[0]+stp*k03,slns[1]+stp*k13)
    slns[0] = slns[0]+stp*(k01+2*k02+2*k03+k04)/6.0
    slns[1] = slns[1]+stp*(k11+2*k12+2*k13+k14)/6.0
    return slns

def rk5(f1,f2,czt,slns,stp):
    k01 = f1(slns[0],slns[1])
    k11 = f2(slns[0],slns[1])

    k02 = f1(slns[0]+stp*k01/4.0,slns[1]+stp*k11/4.0)
    k12 = f2(slns[0]+stp*k01/4.0,slns[1]+stp*k11/4.0)

    k03 = f1(slns[0]+stp*(3.0*k01+9.0*k02)/32.0,slns[1]+stp*(3.0*k11+9.0*k12)/32.0)
    k13 = f2(slns[0]+stp*(3.0*k01+9.0*k02)/32.0,slns[1]+stp*(3.0*k11+9.0*k12)/32.0)

    k04 = f1(slns[0]+stp*(1932*k01-7200.0*k02+7296.0*k03)/2197.0,slns[1]+stp*(1932*k11-7200.0*k12+7296.0*k13)/2197.0)
    k14 = f2(slns[0]+stp*(1932*k01-7200.0*k02+7296.0*k03)/2197.0,slns[1]+stp*(1932*k11-7200.0*k12+7296.0*k13)/2197.0)

    k05 = f1(slns[0]+stp*(439*k01/216.0-8.0*k02+3680.0*k03/513.0-845.0*k04/4104.0),slns[1]+stp*(439*k11/216.0-8.0*k12+3680.0*k13/513.0-845.0*k14/4104.0))
    k15 = f2(slns[0]+stp*(439*k01/216.0-8.0*k02+3680.0*k03/513.0-845.0*k04/4104.0),slns[1]+stp*(439*k11/216.0-8.0*k12+3680.0*k13/513.0-845.0*k14/4104.0))

    k06 = f1(slns[0]+stp*(-8.0*k01/27.0+2.0*k02-3544.0*k03/2565.0+1859.0*k04/4104.0-11.0*k05/40.0),slns[1]+stp*(-8.0*k11/27.0+2.0*k12-3544.0*k13/2565.0+1859.0*k14/4104.0-11.0*k15/40.0))
    k16 = f2(slns[0]+stp*(-8.0*k01/27.0+2.0*k02-3544.0*k03/2565.0+1859.0*k04/4104.0-11.0*k05/40.0),slns[1]+stp*(-8.0*k11/27.0+2.0*k12-3544.0*k13/2565.0+1859.0*k14/4104.0-11.0*k15/40.0))

    slns[0] = slns[0]+stp*(16.0*k01/135.0+6656.0*k03/12825.0+28561.0*k04/56430.0-9.0*k05/50.0+2*k06/55.0)
    slns[1] = slns[1]+stp*(16.0*k11/135.0+6656.0*k13/12825.0+28561.0*k14/56430.0-9.0*k15/50.0+2*k16/55.0)
    return slns

def read_iterdb_file(filename):
    '''
    This Code is Written by: David R. Hatch
    It reads the iterdb file and returns three dictionaries,
    each diectionary has five quantities:
    electron density (NE) and temperature (TE),
    ion density (NM1) and temperatures (TI),
    impurity density (NM2), if any,
    rotational velocity (VROT).
    The three dictionaries provide the toroidal coordinate (rhotor),
    profiles, and units for each quantity.
    '''
    f=open(filename,'r')
    data_in=f.read()
    data_linesplit=data_in.split('\n')

    keep_going=1
    i=0
    while keep_going:
        test=re.search(';-# OF X PTS',data_linesplit[i])
        if test:
            num=data_linesplit[i].split()[0]
            num=float(num)
            num=int(num)
            keep_going=(1==2)
        if i == len(data_linesplit):
            keep_going=(1==2)
        i=i+1

    lnum=0
    try_again=1
    prof_out = {}
    rhot_out = {}
    units_out = {}
    while try_again:
        lnum,try_again,quantity,units,rhot,arr=get_next(data_linesplit,lnum,num)
        prof_out[quantity]=arr
        units_out[quantity]=units
        rhot_out[quantity]=rhot
    return rhot_out,prof_out,units_out

def get_next(data_linesplit,lnum,num):
    sec_num_lines = num/6
    if num % 6 != 0:
        sec_num_lines += 1
    keep_going=1
    while keep_going:
        test=re.search('-DEPENDENT VARIABLE LABEL',data_linesplit[lnum])
        if test :
            quantity=data_linesplit[lnum].split()[0]
            units=data_linesplit[lnum].split()[1]
        test=re.search('DATA FOLLOW',data_linesplit[lnum])
        if test:
            keep_going=(1==2)
        lnum=lnum+1

    rhot=npy.empty(0)
    lnum0 = lnum
    for j in range(lnum0,lnum0+int(sec_num_lines)):
        for k in range(6):
            str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
            if(str_temp):
                temp=npy.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                rhot=npy.append(rhot,temp)
        lnum=lnum+1
    lnum=lnum+1

    arr=npy.empty(0)
    lnum0 = lnum
    for j in range(lnum0,lnum0+int(sec_num_lines)):
        for k in range(6):
            str_temp=data_linesplit[j][1+k*13:1+(k+1)*13]
            if(str_temp):
                temp=npy.array(data_linesplit[j][1+k*13:1+(k+1)*13],dtype='float')
                arr=npy.append(arr,temp)
        lnum=lnum+1

    lnum_out=lnum
    try_again=1
    if len(data_linesplit)-lnum < 10:
        try_again=False
    return lnum_out, try_again,quantity,units,rhot,arr

def plot_iterdb(iterdbdata,reportpath=''):
    if len(reportpath)==0:
       reportpath = './'
    elif 'report' not in reportpath:
       if    reportpath[-1] != "/": reportpath += "/report/"
       else:                        reportpath += "report/"
    rhotor,profiles,units = iterdbdata

    figure = plt.figure('Density')
    axhand = figure.add_subplot(1,1,1)
    axhand.plot(rhotor['NE'],profiles['NE'],label='$n_e$')
    axhand.plot(rhotor['NM1'],profiles['NM1'],label='$n_i$')
    if len(rhotor.keys()) >= 6:
       axhand.plot(rhotor['NM2'],profiles['NM2'],label='$n_z$')
    axhand.set_title('Density Profiles')
    axhand.set_xlabel('$\\rho_{tor}$')
    axhand.set_ylabel('Density('+units['NE']+')')
    axhand.legend()
    figure.savefig(reportpath+'iterdb_density.png') 
    plt.close(figure)

    figure = plt.figure('Temperature')
    axhand = figure.add_subplot(1,1,1)
    axhand.plot(rhotor['TE'],profiles['TE'],label='$T_e$')
    axhand.plot(rhotor['TI'],profiles['TI'],label='$T_i$')
    axhand.set_title('Temperature Profiles')
    axhand.set_xlabel('$\\rho_{tor}$')
    axhand.set_ylabel('Temperature('+units['TE']+')')
    axhand.legend()
    figure.savefig(reportpath+'iterdb_temperature.png') 
    plt.close(figure)

    return 1

def read_profiles_file(pfpath,setParam={}):
   #Developed by Ehab Hassan on 2019-02-17
    from scipy.interpolate import CubicSpline
    if not os.path.isfile(pfpath):
       print('Fatal: file %s not found.' % pfpath)
       sys.exit()

    ofh = open(pfpath,'r')

    profiles = {}
    units    = {}
    while True:
          recs = ofh.readline().split()
          if   len(recs)>4:
               nrec = int(recs[0])
               ary0=npy.zeros(nrec)
               ary1=npy.zeros(nrec)
               ary2=npy.zeros(nrec)
               var0 = str(recs[1]).lower()
               var1 = str(recs[2]).lower()
               var2 = str(recs[3]).lower()
               for i in range(nrec):
                   recs = ofh.readline().split()
                   ary0[i] = float(recs[0])
                   ary1[i] = float(recs[1])
                   ary2[i] = float(recs[2])
               profiles[var0]=ary0
               profiles[var1]=ary1
               profiles[var2]=ary2
          elif len(recs)==4:
               nrec = int(recs[0])
               ary0=npy.zeros(nrec)
               ary1=npy.zeros(nrec)
               ary2=npy.zeros(nrec)
               var0 = str(recs[1]).lower()
               temp = str(recs[2]).lower()
               var1 = temp[:temp.index("(")]
               if   var1.strip() in ['ne','ni','nb','nz1']:
                    powr = int(temp[temp.index("^")+1:temp.index("/")])
                    unit = 'm^{-3}'
               elif var1.strip() in ['te','ti']:
                    unit = 'eV'
               elif var1.strip() in ['pb','ptot']:
                    unit = 'Pa'
               elif var1.strip() in ['vtor1','vpol1']:
                    unit = 'm/s'
               else:
                    unit = temp[temp.index("(")+1:temp.index(")")]
               var2 = str(recs[3]).lower()
               for i in range(nrec):
                   recs    = ofh.readline().split()
                   ary0[i] = float(recs[0])
                   ary1[i] = float(recs[1])
                   ary2[i] = float(recs[2])
               if var0 in profiles.keys():
                  CS = CubicSpline(ary0,ary1)
                  units[var1]    = unit
                  profiles[var1] = CS(profiles[var0])
                  CS = CubicSpline(ary0,ary2)
                  profiles[var2] = CS(profiles[var0])
               else:
                  profiles[var0] = ary0
                  units[var1]    = unit
                  profiles[var1] = ary1
                  profiles[var2] = ary2
               if   var1.strip() in ['ne','ni','nb','nz1']:
                    profiles[var1] *= 10.0**powr
               elif var1.strip() in ['te','ti','pb','ptot','vtor1','vpol1']:
                    profiles[var1] *= 1.0e3
          else:
               break
    ofh.close()

    if 'rhotor' in setParam:
       if 'eqdskfpath' not in setParam:
          eqdskfpath = raw_input('Path to EQDSK file: ')
       elif 'eqdskfpath' in setParam:
          eqdskfpath = setParam['eqdskfpath']

       eqdskdata = read_efit_file(eqdskfpath)

       qpsifn  = interp1d(eqdskdata['PSIN'],eqdskdata['qpsi'])
       qpsi    = qpsifn(profiles['psinorm'])
       qpsifn  = interp1d(profiles['psinorm'],qpsi)
       psinorm = npy.linspace(profiles['psinorm'][0],profiles['psinorm'][-1],setParam['rhotor'])
       qpsi    = qpsifn(psinorm)
       phinorm = npy.zeros_like(psinorm)
       for i in range(1,npy.size(qpsi)):
           x = psinorm[:i+1]
           y = qpsi[:i+1]
           phinorm[i] = npy.trapz(y,x)
       profiles['rhotor'] = npy.sqrt(phinorm)
       
    return profiles,units

def plot_profiles(pfpath):
    from matplotlib.backends.backend_pdf import PdfPages
    profiles = read_profiles(pfpath)
    nvars = sorted(profiles.keys())
    print('Plotting profiles in: %s ...' % pfpath)
    figs  = PdfPages('profiles.pdf')
    for i in nvars:
        if i in ['n','z','a','psinorm']: continue
        figname = plt.figure(i)
        plt.plot(profiles['psinorm'],profiles[i])
        plt.xlabel('$\Psi$')
        plt.ylabel(i)
        figs.savefig(figname)
    figs.close()
    return profiles

def read_efit_file(eqdskfpath,setParam={}):
   #Developed by Ehab Hassan on 2019-02-27
    if os.path.isfile(eqdskfpath) == False:
       errorFunc = traceback.extract_stack(limit=2)[-2][3]
       errorLine = traceback.extract_stack(limit=2)[-2][1]
       errorFile = traceback.extract_stack(limit=2)[-2][2]
       errMSG    = 'Call %s line %5d in file %s Failed.\n'
       errMSG   += 'Fatal: file %s not found.'
       raise IOError(errMSG %(errorFunc,errorLine,errorFile,eqdskfpath))
    
    ofh = open(eqdskfpath,'r')
    eqdskdata = {}
    cline = ofh.readline()
    eqdskdata['idum']   = int(cline[48:52])
    eqdskdata['RDIM']   = int(cline[52:56])
    eqdskdata['ZDIM']   = int(cline[56:61])
    cline = ofh.readline()
    eqdskdata['RLEN']   = float(cline[0:16])
    eqdskdata['ZLEN']   = float(cline[16:32])
    eqdskdata['RCTR']   = float(cline[32:48])
    eqdskdata['RLFT']   = float(cline[48:64])
    eqdskdata['ZMID']   = float(cline[64:80])
    cline = ofh.readline()
    eqdskdata['RMAX']   = float(cline[0:16])
    eqdskdata['ZMAX']   = float(cline[16:32])
    eqdskdata['PSIMAX'] = float(cline[32:48])
    eqdskdata['PSIBND'] = float(cline[48:64])
    eqdskdata['BCTR']   = float(cline[64:80])
    cline = ofh.readline()
    eqdskdata['CURNT']  = float(cline[0:16])
    eqdskdata['PSIMAX'] = float(cline[16:32])
    eqdskdata['XDUM']   = float(cline[32:48])
    eqdskdata['RMAX']   = float(cline[48:64])
    eqdskdata['XDUM']   = float(cline[64:80])
    cline = ofh.readline()
    eqdskdata['ZMAX']   = float(cline[0:16])
    eqdskdata['XDUM']   = float(cline[16:32])
    eqdskdata['PSIBND'] = float(cline[32:48])
    eqdskdata['XDUM']   = float(cline[48:64])
    eqdskdata['XDUM']   = float(cline[64:80])

    nlines1D = int(npy.ceil(eqdskdata['RDIM']/5.0))

    eqdskdata['fpol'] = npy.zeros(eqdskdata['RDIM'])
    for iline in range(nlines1D):
        cline = ofh.readline()
        try:
            eqdskdata['fpol'][iline*5+0] = float(cline[0:16])
            eqdskdata['fpol'][iline*5+1] = float(cline[16:32])
            eqdskdata['fpol'][iline*5+2] = float(cline[32:48])
            eqdskdata['fpol'][iline*5+3] = float(cline[48:64])
            eqdskdata['fpol'][iline*5+4] = float(cline[64:80])
        except:
            error = 'empty records'

    eqdskdata['pressure'] = npy.zeros(eqdskdata['RDIM'])
    for iline in range(nlines1D):
        cline = ofh.readline()
        try:
            eqdskdata['pressure'][iline*5+0] = float(cline[0:16])
            eqdskdata['pressure'][iline*5+1] = float(cline[16:32])
            eqdskdata['pressure'][iline*5+2] = float(cline[32:48])
            eqdskdata['pressure'][iline*5+3] = float(cline[48:64])
            eqdskdata['pressure'][iline*5+4] = float(cline[64:80])
        except:
            error = 'empty records'

    eqdskdata['ffprime'] = npy.zeros(eqdskdata['RDIM'])
    for iline in range(nlines1D):
        cline = ofh.readline()
        try:
            eqdskdata['ffprime'][iline*5+0] = float(cline[0:16])
            eqdskdata['ffprime'][iline*5+1] = float(cline[16:32])
            eqdskdata['ffprime'][iline*5+2] = float(cline[32:48])
            eqdskdata['ffprime'][iline*5+3] = float(cline[48:64])
            eqdskdata['ffprime'][iline*5+4] = float(cline[64:80])
        except:
            error = 'empty records'

    eqdskdata['pprime'] = npy.zeros(eqdskdata['RDIM'])
    for iline in range(nlines1D):
        cline = ofh.readline()
        try:
            eqdskdata['pprime'][iline*5+0] = float(cline[0:16])
            eqdskdata['pprime'][iline*5+1] = float(cline[16:32])
            eqdskdata['pprime'][iline*5+2] = float(cline[32:48])
            eqdskdata['pprime'][iline*5+3] = float(cline[48:64])
            eqdskdata['pprime'][iline*5+4] = float(cline[64:80])
        except:
            error = 'empty records'

    nlines2D = int(npy.ceil(eqdskdata['RDIM']*eqdskdata['ZDIM']/5.0))

    eqdskdata['psiRZ'] = npy.zeros(eqdskdata['RDIM']*eqdskdata['ZDIM'])
    for iline in range(nlines2D):
        cline = ofh.readline()
        try:
            eqdskdata['psiRZ'][iline*5+0] = float(cline[0:16])
            eqdskdata['psiRZ'][iline*5+1] = float(cline[16:32])
            eqdskdata['psiRZ'][iline*5+2] = float(cline[32:48])
            eqdskdata['psiRZ'][iline*5+3] = float(cline[48:64])
            eqdskdata['psiRZ'][iline*5+4] = float(cline[64:80])
        except:
            error = 'empty records'
    eqdskdata['psiRZ'] = npy.reshape(eqdskdata['psiRZ'],(eqdskdata['ZDIM'],eqdskdata['RDIM']))

    eqdskdata['qpsi'] = npy.zeros(eqdskdata['RDIM'])
    for iline in range(nlines1D):
        cline = ofh.readline()
        try:
            eqdskdata['qpsi'][iline*5+0] = float(cline[0:16])
            eqdskdata['qpsi'][iline*5+1] = float(cline[16:32])
            eqdskdata['qpsi'][iline*5+2] = float(cline[32:48])
            eqdskdata['qpsi'][iline*5+3] = float(cline[48:64])
            eqdskdata['qpsi'][iline*5+4] = float(cline[64:80])
        except:
            error = 'empty records'
 
    cline = ofh.readline()
    eqdskdata['nbound'] = int(cline[0:5])
    eqdskdata['nlimit'] = int(cline[5:10])

    if eqdskdata['nbound'] > 0:
       nlines1D = int(npy.ceil(2*eqdskdata['nbound']/5.0))

       Ary1D = npy.zeros(2*eqdskdata['nbound'])
       for iline in range(nlines1D):
           cline = ofh.readline()
           try:
               Ary1D[iline*5+0] = float(cline[0:16])
               Ary1D[iline*5+1] = float(cline[16:32])
               Ary1D[iline*5+2] = float(cline[32:48])
               Ary1D[iline*5+3] = float(cline[48:64])
               Ary1D[iline*5+4] = float(cline[64:80])
           except:
               error = 'empty records'

       eqdskdata['rbound'] = Ary1D[0::2]
       eqdskdata['zbound'] = Ary1D[1::2]


    if eqdskdata['nlimit'] > 0:
       nlines1D = int(npy.ceil(2*eqdskdata['nlimit']/5.0))

       Ary1D = npy.zeros(2*eqdskdata['nlimit'])
       for iline in range(nlines1D):
           cline = ofh.readline()
           try:
               Ary1D[iline*5+0] = float(cline[0:16])
               Ary1D[iline*5+1] = float(cline[16:32])
               Ary1D[iline*5+2] = float(cline[32:48])
               Ary1D[iline*5+3] = float(cline[48:64])
               Ary1D[iline*5+4] = float(cline[64:80])
           except:
               error = 'empty records'

       eqdskdata['rlimit'] = Ary1D[0::2]
       eqdskdata['zlimit'] = Ary1D[1::2]

 
    eqdskdata['ZR1D']  = npy.arange(eqdskdata['ZDIM'],dtype=float)*eqdskdata['ZLEN']/(eqdskdata['ZDIM']-1.0)
    eqdskdata['ZR1D'] += eqdskdata['ZMID']-eqdskdata['ZMID']/2.0

    eqdskdata['RR1D']  = npy.arange(eqdskdata['RDIM'],dtype=float)*eqdskdata['RLEN']/(eqdskdata['RDIM']-1.0)
    eqdskdata['RR1D'] += eqdskdata['RLFT']

    eqdskdata['psiRZ'] = (eqdskdata['psiRZ']-eqdskdata['PSIMAX'])/(eqdskdata['PSIBND']-eqdskdata['PSIMAX'])

    eqdskdata['PSI']    = (eqdskdata['PSIBND']-eqdskdata['PSIMAX'])*npy.arange(eqdskdata['RDIM'])/(eqdskdata['RDIM']-1.0)
    eqdskdata['PSIN']   = (eqdskdata['PSI']-eqdskdata['PSI'][0])/(eqdskdata['PSI'][-1]-eqdskdata['PSI'][0])
    eqdskdata['rhopsi'] = npy.sqrt(eqdskdata['PSIN'])

    extendPSI    = npy.linspace(eqdskdata['PSI'][0],eqdskdata['PSI'][-1],10*npy.size(eqdskdata['PSI']))
    extendPHI    = npy.empty_like(extendPSI)
    extendPHI[0] = 0.0
    qfunc        = CubicSpline(eqdskdata['PSI'],eqdskdata['qpsi'])
    for i in range(1,npy.size(extendPSI)):
        x           = extendPSI[:i+1]
        y           = qfunc(x)
        extendPHI[i]= npy.trapz(y,x)

    eqdskdata['PHI'] = npy.empty_like(eqdskdata['PSI'])
    phifunc          = CubicSpline(extendPSI,extendPHI)
    for i in range(npy.size(eqdskdata['PSI'])):
        eqdskdata['PHI'][i] = phifunc(eqdskdata['PSI'][i])

    eqdskdata['PHIN']   = (eqdskdata['PHI']-eqdskdata['PHI'][0])/(eqdskdata['PHI'][-1]-eqdskdata['PHI'][0])
    eqdskdata['rhotor'] = npy.sqrt(eqdskdata['PHIN'])

    return eqdskdata

def plot_eqdsk(eqdskdata,reportpath=''):
   #Developed by Ehab Hassan on 2019-03-11
    if len(reportpath)==0:
       reportpath = './'
    else:
       if    reportpath[-1] != "/": reportpath += "/report/"
       else:                        reportpath += "report/"

    figure = plt.figure('Magnetic Surface Boundary')
    axhand = figure.add_subplot(1,1,1)
    axhand.plot(eqdskdata['rbound'],eqdskdata['zbound'])
    axhand.set_title('Magnetic Surface Boundary')
    axhand.set_xlabel('r')
    axhand.set_ylabel('z')
    figure.savefig(reportpath+'eqdsk_magsurfbound.png')
    plt.close(figure)

    figure = plt.figure('Safety Factor')
    axhand = figure.add_subplot(1,1,1)
    axhand.plot(eqdskdata['rhopsi'],eqdskdata['qpsi'])
    axhand.set_title('Safety Factor')
    axhand.set_xlabel('$\\rho_{\\psi}$')
    axhand.set_ylabel('q')
    figure.savefig(reportpath+'eqdsk_safetyfactor.png')
    plt.close(figure)

    figure = plt.figure('Pressure Gradient')
    axhand = figure.add_subplot(1,1,1)
    axhand.plot(eqdskdata['rhopsi'],eqdskdata['pprime'])
    axhand.set_title('Pressure Gradient')
    axhand.set_xlabel('$\\rho_{\\psi}$')
    axhand.set_ylabel("P'")
    figure.savefig(reportpath+'eqdsk_pprime.png')
    plt.close(figure)

    figure = plt.figure('Current Flux')
    axhand = figure.add_subplot(1,1,1)
    axhand.plot(eqdskdata['rhopsi'],eqdskdata['ffprime'])
    axhand.set_title('Cuurent Flux')
    axhand.set_xlabel('$\\rho_{\\psi}$')
    axhand.set_ylabel("FF'")
    figure.savefig(reportpath+'eqdsk_ffprime.png')
    plt.close(figure)

    return 1

def profile_mapping(psi,phi,profpsi=[],profphi=[]):
    if   any(profpsi):
         profpsifn=interp1d(psi,profpsi,kind='linear')
         phipsifn =interp1d(psi,phi,kind='linear')
         profphi  =profpsifn(phipsifn(psi))
    elif any(profphi):
         profphifn=interp1d(phi,profphi,kind='linear')
         psiphifn =interp1d(phi,psi,kind='linear')
         profpsi  =profphifn(psiphifn(phi))

def psi2phi(q,psi):
    qpsifn  = interp1d(psi,q)
    psinorm = npy.linspace(psi[0],psi[-1],10*len(psi))
    qpsi    = qpsifn(psinorm)
    phinorm = npy.zeros_like(psinorm)
    for i in range(1,npy.size(qpsi)):
        x = psinorm[1:i+1]
        y = qpsi[1:i+1]
        phinorm[i] = npy.trapz(y,x)
    phinorm  = (phinorm-phinorm[0])/(phinorm[-1]-phinorm[0])
    phipsifn = interp1d(psinorm,phinorm)
    phi      = phipsifn(psi)
    return phi

def phi2psi(q,phi):
    qphifn  = interp1d(phi,q)
    phinorm = npy.linspace(phi[0],phi[-1],10*len(phi))
    qphi    = qphifn(phinorm)
    psinorm = npy.zeros_like(phinorm)
    for i in range(1,npy.size(qphi)):
        x = phinorm[1:i+1]
        y = 1./qphi[1:i+1]
        psinorm[i] = npy.trapz(y,x)
    psinorm  = (psinorm-psinorm[0])/(psinorm[-1]-psinorm[0])
    psiphifn = interp1d(phinorm,psinorm)
    psi      = psiphifn(phi)
    return psi


def findmonotonic(A,kind="increasing"):
    if kind.lower()=="increasing":
       bgnloc=0
       endloc=len(A)-1
       for i in range(len(A)):
           if A[i+1]>A[i]:
              bgnloc=i
              break
       for i in range(bgnloc,len(A)-1):
           if A[i+1]<A[i]:
              endloc=i
              break
    return bgnloc,endloc


def magsurf_contours(eqdskfpath):
    eqdskdata = read_efit_file(eqdskfpath.strip())

    minZ = min(eqdskdata['zbound'])+eqdskdata['ZLEN']/2
    maxZ = max(eqdskdata['zbound'])+eqdskdata['ZLEN']/2
    minR = min(eqdskdata['rbound'])
    maxR = max(eqdskdata['rbound'])

    R2D,Z2D = npy.meshgrid(eqdskdata['RR1D'],eqdskdata['ZR1D'])

    fig,ax = plt.subplots(1)
    cs = plt.contour(R2D,Z2D,eqdskdata['psiRZ'],1000)
    plt.close(fig)
    lines = []
    for j in range(0,len(cs.levels)):
      if cs.levels[j]>1.0:
         break
      else:
         for line in cs.collections[j].get_paths():
           lines.append(line.vertices)
           if any(lines[-1][:,0] <= minR):  continue
           if any(lines[-1][:,0] >= maxR):  continue
           if any(lines[-1][:,1] <= minZ):  continue
           if any(lines[-1][:,1] >= maxZ):  continue
           rcval = lines[-1][:,0]
           zcval = lines[-1][:,1]
    return rcval,zcval-eqdskdata['ZLEN']/2

def magsurf_solvflines(eqdskfpath='',eqdskdata={},psi=1.0,eps=1.0e-16):
    if not eqdskdata.keys():
       if eqdskfpath:
          eqdskdata = read_efit_file(eqdskfpath.strip())
       else:
          print('FATAL: EQDSK FILE NOT FOUND. EXIT!')
          sys.exit()

    qpsifn = interp1d(eqdskdata['PSIN'],eqdskdata['qpsi'],kind="linear")

    minZ = min(eqdskdata['zbound'])+eqdskdata['ZLEN']/2
    maxZ = max(eqdskdata['zbound'])+eqdskdata['ZLEN']/2
    minR = min(eqdskdata['rbound'])
    maxR = max(eqdskdata['rbound'])

    psiRZ    = interp2d(eqdskdata['RR1D'],eqdskdata['ZR1D'],eqdskdata['psiRZ'],kind="cubic")
    dpsiRZdR = psiRZ.__call__(x=eqdskdata['RR1D'],y=eqdskdata['ZR1D'],dx=1)
    dpsiRZdZ = psiRZ.__call__(x=eqdskdata['RR1D'],y=eqdskdata['ZR1D'],dy=1)
    BR       =-dpsiRZdZ/eqdskdata['BCTR']/eqdskdata['RCTR']
    BZ       = dpsiRZdR/eqdskdata['BCTR']/eqdskdata['RCTR']
    f1       = interp2d(eqdskdata['RR1D'],eqdskdata['ZR1D'],BR,kind="cubic")
    f2       = interp2d(eqdskdata['RR1D'],eqdskdata['ZR1D'],BZ,kind="cubic")

    cs_psi = 1.0
    for k in range(npy.size(eqdskdata['ZR1D'])):
        z    = eqdskdata['ZR1D'][npy.size(eqdskdata['ZR1D'])-k-1]
        psir = interp1d(eqdskdata['RR1D'],psiRZ(eqdskdata['RR1D'],z),kind="cubic")
        r    = bisection(psir,eqdskdata['RR1D'][0],eqdskdata['RR1D'][-1],root=psi,eps=eps)
        if r <= minR or r >= maxR:  continue
        if z <= minZ or z >= maxZ:  continue
        if abs(psiRZ(r,z)-psi)<eps or abs(psiRZ(r,z)-psi)<cs_psi:
           cs_psi = abs(psiRZ(r,z)-psi)
           cs_r   = r
           cs_z   = z

    npts = 1024

    rsln = []
    zsln = []
    slns = npy.array([cs_r,cs_z])

    if type(psiRZ(cs_r,cs_z))==float:
       maxzta = 4.0*npy.pi*eqdskdata['RLEN']*eqdskdata['ZLEN']*psiRZ(cs_r,cs_z)
    else:
       maxzta = 4.0*npy.pi*eqdskdata['RLEN']*eqdskdata['ZLEN']*max(psiRZ(cs_r,cs_z))
    zta = npy.linspace(0.0,maxzta,npts)
    stp = npy.diff(zta)[0]
    rflipped = 0; rflag = False
    zflipped = 0; zflag = False
    nzta = npy.size(zta)
    for j in range(nzta):
       slns = rk5(f1,f2,zta[j],slns,stp)
       if rflipped >= 2: rflag = True
       if zflipped >= 2: zflag = True
       if (rflag and zflag) and (abs(rref-slns[0])<0.05 and abs(zref-slns[1])<0.05):
          break
       rsln.append(slns[0])
       zsln.append(slns[1])
       if j==0:
          rref = eqdskdata['rbound'][npy.argmin(abs(rsln[0]-eqdskdata['rbound']))]
          zref = eqdskdata['zbound'][npy.argmin(abs(zsln[0]-eqdskdata['zbound']))]
       elif j==1:
          rrefsign = npy.sign(rsln[-1]-rsln[-2])
          zrefsign = npy.sign(zsln[-1]-zsln[-2])
       elif j>2:
          rcrntsign = npy.sign(rsln[-1]-rsln[-2])
          zcrntsign = npy.sign(zsln[-1]-zsln[-2])
          if rrefsign ==-rcrntsign: rflipped += 1; rrefsign = rcrntsign 
          if zrefsign ==-zcrntsign: zflipped += 1; zrefsign = zcrntsign 

    return npy.array(rsln),npy.array(zsln)-eqdskdata['ZLEN']/2

def magsurf_searching(eqdskfpath='',eqdskdata={},psi=1.0,eps=1.0e-6):
    if not eqdskdata.keys():
       if eqdskfpath:
          eqdskdata = read_efit_file(eqdskfpath.strip())
       else:
          print('FATAL: EQDSK FILE NOT FOUND. EXIT!')
          sys.exit()

    minZ = min(eqdskdata['zbound'])+eqdskdata['ZLEN']/2
    maxZ = max(eqdskdata['zbound'])+eqdskdata['ZLEN']/2
    minR = min(eqdskdata['rbound'])
    maxR = max(eqdskdata['rbound'])

    psiRZ    = interp2d(eqdskdata['RR1D'],eqdskdata['ZR1D'],eqdskdata['psiRZ'],kind="cubic")

    rbound = []
    zbound = []

#  #Z1D = npy.linspace(eqdskdata['ZR1D'][0],eqdskdata['ZR1D'][-1],1024)
#  #R1D = npy.linspace(minR,maxR,1024)
#  #R1D = npy.linspace(eqdskdata['RR1D'][0],eqdskdata['RR1D'][-1],1024)

#   Z1D = npy.linspace(minZ,maxZ,1024)
#  #medR= (minR+maxR)/2.0
#  #R1D = npy.linspace(medR,maxR,1024)
#   R1D = npy.linspace(eqdskdata['RR1D'][0],eqdskdata['RR1D'][npy.size(eqdskdata['RR1D'])/2] ,1024)
#   for k in range(npy.size(Z1D)):
#       z    = Z1D[k]
#       psir = interp1d(R1D,psiRZ(R1D,z),kind="cubic")
#       r    = bisection(psir,R1D[0],R1D[-1],root=psi,eps=eps)
#       if r <= minR or r >= maxR:  continue
#       if z <= minZ or z >= maxZ:  continue
#       if abs(psiRZ(r,z)-psi)<eps:
#          rbound.append(r)
#          zbound.append(z)

#  #R1D = npy.linspace(minR,medR,1024)
#   R1D = npy.linspace(eqdskdata['RR1D'][npy.size(eqdskdata['RR1D'])/2],eqdskdata['RR1D'][-1],1024)
#   for k in range(npy.size(Z1D)):
#       z    = Z1D[k]
#       psir = interp1d(R1D,psiRZ(R1D,z),kind="cubic")
#       r    = bisection(psir,R1D[0],R1D[-1],root=psi,eps=eps)
#       if r <= minR or r >= maxR:  continue
#       if z <= minZ or z >= maxZ:  continue
#       if abs(psiRZ(r,z)-psi)<eps:
#          rbound.append(r)
#          zbound.append(z)

    ZMAX = eqdskdata['ZMAX']
    RMAX = eqdskdata['RMAX']
    dens = 512
    for i in range(npy.size(eqdskdata['rbound'])):
        rho = npy.sqrt((eqdskdata['rbound'][i]-RMAX)**2+(eqdskdata['zbound'][i]-ZMAX)**2)
        ang = npy.arctan2(eqdskdata['zbound'][i]-ZMAX,eqdskdata['rbound'][i]-RMAX)
        rho1D = npy.linspace(0.0,rho,dens)
        psi1D = npy.zeros(dens)
        for irho in range(dens):
            r = RMAX+rho1D[irho]*npy.cos(ang)
            z = ZMAX+rho1D[irho]*npy.sin(ang)
            psi1D[i] = psiRZ(r,z)
        psirho = interp1d(rho1D,psi1D,kind="cubic")
        rhopsi = bisection(psirho,rho1D[0],rho1D[-1],root=psi,eps=eps)
        if abs(psirho(rhopsi)-psi) <= eps:
           rbound.append(RMAX+rhopsi*npy.cos(ang))
           zbound.append(ZMAX+rhopsi*npy.sin(ang))

    return npy.array(rbound),npy.array(zbound)

def magsurf_interp(eqdskfpath):
    eqdskdata = read_efit_file(eqdskfpath.strip())

   #Finding the magnetic surface using interpolation technique
    R1D = eqdskdata['RR1D']-eqdskdata['RLFT']-(eqdskdata['RLEN']/2.0)
    Z1D = eqdskdata['ZR1D']-(eqdskdata['ZLEN']/2.0)
    psiRZfn  = interp2d(R1D,Z1D,eqdskdata['psiRZ'],kind="linear")

    rho1D = npy.sqrt(R1D[npy.size(R1D)/2:]**2+Z1D[npy.size(Z1D)/2:]**2)
    ang1D = npy.linspace(0.0,2.0*npy.pi,npy.size(rho1D))

    pct      = 1.00000
    psi      = (eqdskdata['PSIBND']-eqdskdata['PSIMAX'])*npy.arange(npy.size(rho1D))/(npy.size(rho1D)-1.0)
    psiN     = ((psi-psi[0])/(psi[-1]-psi[0]))*pct

    rho2D    = npy.zeros(npy.size(psiN))
    nwR2D    = npy.zeros(npy.size(psiN))
    nwZ2D    = npy.zeros(npy.size(psiN))

    for j in range(npy.size(ang1D)):
        nwpsi = []
        nwrho = []
        for i in range(npy.size(rho1D)):
            r = rho1D[i]*npy.cos(ang1D[j])
            z = rho1D[i]*npy.sin(ang1D[j])
            psival = psiRZfn(r,z)
            if psival[0] <= 1.0:
               nwpsi.append(psival[0])
               nwrho.append(rho1D[i])
            else:
               continue
        nwpsi = npy.array(nwpsi)
        nwrho = npy.array(nwrho)
        bgnloc,endloc=findmonotonic(nwpsi)
        psifn = interp1d(nwpsi[bgnloc:endloc+1],nwrho[bgnloc:endloc+1],kind="linear",fill_value="extrapolate")
        rho2D = npy.vstack((rho2D,psifn(psiN)))
        nwR2D = npy.vstack((nwR2D,rho2D[-1,:]*npy.cos(ang1D[j])))
        nwZ2D = npy.vstack((nwZ2D,rho2D[-1,:]*npy.sin(ang1D[j])))

    rho2D = npy.delete(rho2D,0,0)
    nwR2D = npy.delete(nwR2D,0,0)
    nwZ2D = npy.delete(nwZ2D,0,0)

    return nwR2D[:,-1].nwZ2D[-1,:]




from .Tools.DispersionRelationDeterminantFullConductivityZeff import VectorFinder_auto
from .Tools.DispersionRelationDeterminantFullConductivityZeff import VectorFinder_manual
from .Tools.DispersionRelationDeterminantFullConductivityZeff import VectorFinder_auto_Extensive


def Dispersion(nu,zeff,eta,shat,beta,ky,mu,xstar,Run_mode=1):
    
    if Run_mode==1: #fast mode
        w0=1.+eta   #frequency = ome = 1+eta
    if Run_mode==2: #slow mode (global)
        w0=VectorFinder_auto(nu,zeff,eta,shat,beta,ky,1,abs(mu),xstar)
    if Run_mode==3: #slow mode (local)
        w0=VectorFinder_auto(nu,zeff,eta,shat,beta,ky,0,abs(mu),xstar)
    elif Run_mode==4: #manual mode
        w0=VectorFinder_manual(nu,zeff,eta,shat,beta,ky,1,abs(mu),xstar)
    elif Run_mode==5: #slow slow mode (global)
        w0=VectorFinder_auto_Extensive(nu,zeff,eta,shat,beta,ky,1,abs(mu),xstar)
    elif Run_mode==6: #NN mode
        from .SLiM_NN.Dispersion_NN import Dispersion_NN
        Dispersion_NN_obj=Dispersion_NN()
        w0=Dispersion_NN_obj.Dispersion_omega(nu,zeff,eta,shat,beta,ky,abs(mu),xstar)

    return w0



def Dispersion_list(para_list,Run_mode=1):
    
    if Run_mode==6: #NN mode
        from .SLiM_NN.Dispersion_NN import Dispersion_NN
        Dispersion_NN_obj=Dispersion_NN(path)
        w0_list=[]
        for [nu,zeff,eta,shat,beta,ky,mu,xstar] in para_list:
            w0=Dispersion_NN_obj.Dispersion_omega(nu,zeff,eta,shat,beta,ky,mu,xstar)
            w0_list.append(w0)
    else:
        for [nu,zeff,eta,shat,beta,ky,mu,xstar] in para_list:
            w0=Dispersion(nu,zeff,eta,shat,beta,ky,mu,xstar,Run_mode)
            w0_list.append(w0)

    return w0_list




from .read_profiles import read_geom_file
import numpy as np 


def mod_q_gfile(original_geofile_name,output_geofile_name,x_output,q_output,show_plot=False):
    xgrid, q_original, R_ref= read_geom_file('gfile',original_geofile_name,'.dat')

    f=open(original_geofile_name,'r')
    data = f.read()
    f.close()

    sdata = data.split('\n')

    #check the line # for different quantity 
    quant_dic={}
    for line,i in zip(sdata,range(len(sdata))):
        line=line.split()
        try:
            if '=' in line[1]:
                quant_dic[line[0]]=i
        except:
            pass

    keys=list(quant_dic.keys())
    
    print(keys)

    start_index=0
    end_index=0
    for key,i in zip(keys,range(len(keys))):
        if 'QPSI' == key:
            print(key)
            start_index=quant_dic[keys[i]]
            end_index=quant_dic[keys[i+1]]

    print('start_index='+str(start_index))
    print('end_index='+str(end_index))


    #read q
    if 1==0:
        q_data=[]
        for i in sdata[start_index].split()[2:]:
            q_data.append(float(i))
        for i in range(start_index+1,end_index):
            for j in sdata[i].split():
                q_data.append(float(j))



    #start of write the q
    file=open(output_geofile_name,"w")


    #before the q 
    for i in range(start_index):
        file.write(sdata[i]+'\n')

    q_output_tmp = np.interp(xgrid,x_output,q_output)

    #output q
    #first line
    file.write(' QPSI =    %.15f         %.15f      \n'%(q_output_tmp[0],q_output_tmp[1]))
    
    #the rest of the lines

    q_output_rest=q_output_tmp[2:]
    print('len(q_output_rest)')
    print(len(q_output_rest))

    if show_plot==True:
        import matplotlib.pyplot as plt
        plt.clf()
        plt.plot(xgrid,q_output_tmp,label='output')
        plt.plot(xgrid,q_original,label='from_gfile')
        plt.legend()
        plt.show()

    line_tmp=''
    for i in range(len(q_output_rest)):
        if i%3==0:#1st quantitity in the row
            line_tmp=line_tmp+('    %.15f'%q_output_rest[i])
        elif i%3==1:#2nd quantitity in the row
            line_tmp=line_tmp+('         %.15f'%q_output_rest[i])
        elif i%3==2:#3rd quantitity in the row
            line_tmp=line_tmp+('         %.15f      \n'%q_output_rest[i])
            file.write(line_tmp)
            line_tmp=''
    if len(q_output_rest)%3!=0:
        file.write(line_tmp+'      \n')


    #After the q 
    for i in range(end_index,len(sdata)):
        file.write(sdata[i]+'\n')



    



    


""" ParIO.py: Contains the class to handle reading and writing of parameter files """
import re
import sys
from collections import namedtuple, OrderedDict


class Parameters(object):
    """ Parameters class:

    Converts a GENE parameters file to a python dictionary and vice versa
    """
    def __init__(self):
        # dictionary for parameters: {parameter: value}
        self.pardict = OrderedDict()
        # dictionary for recording the namelists the parameters belong to: {parameter: namelist}
        self.nmldict = OrderedDict()
        # keep track of all namelists that have been found
        self.namelists = []
        self.spec_nl = ('omn', 'omt', 'mass', 'charge', 'dens', 'temp', 'name', 'passive',
                        'kappa_n', 'kappa_T', 'LT_center', 'Ln_center', 'LT_width', 'Ln_width',
                        'prof_type', 'src_prof_type', 'src_amp', 'src_width', 'src_x0',
                        'delta_x_n', 'delta_x_T' , 'prof_file')
        self.boolstr_t = [".T.", ".t.", "T", "t", ".true."]
        self.boolstr_f = [".F.", ".f.", "F", "f", ".false."]

    @staticmethod
    def clearcomments(variable):
        regex = re.compile(r'\s*([-+\'\"\[\];.,/a-zA-Z0-9_\s*]*)\s*!?\s*(.*)')
        result = regex.search(variable)
        if result and result.group(2)[:4] != 'scan':
            return regex.search(variable).group(1)
        else:
            return variable

    def Read_Pars(self, path):
        """ Read parameters file and make it a dict """
        # counts species namelists
        countspec = 0
        try:
            with open(path, "r") as parfile:
                # Search file for parameters using regular expressions
                for line in parfile:
                    # Exclude commented lines
                    if re.search(r'\s*!\w*\s*=.*', line) is None:
                        # Check for and count species namelists
                        if re.search(r'^\s*&(.*)', line):
                            # if namelist belongs to a species, append its number to the namelist
                            if re.search(r'^\s*&(.*)', line).group(1) == 'species':
                                countspec += 1
                                nml = re.search(r'^\s*&(.*)', line).group(1)+str(countspec)
                            else:
                                nml = re.search(r'^\s*&(.*)', line).group(1)
                            if nml not in self.namelists:
                                self.namelists.append(nml)
                    # Search lines for <parameter> = <value> patterns
                    p = re.compile(r'^\s*(.*)\s*=\s*(.*)')
                    m = p.search(line)
                    # Pick matching lines and build parameter dictionary
                    if m:
                        if m.group(1).strip() in self.spec_nl:
                            self.pardict[m.group(1).strip()+str(countspec)] = m.group(2)
                            self.nmldict[m.group(1).strip()+str(countspec)] = nml
                        else:
                            self.pardict[m.group(1).strip()] = m.group(2)
                            self.nmldict[m.group(1).strip()] = nml
        except IOError:
            sys.exit("ParIO: ReadPars: could not read parameters file")
        # clear the comments from all variables, cast some strings to integers and floats
        for item in self.pardict:
            self.pardict[item] = self.clearcomments(self.pardict[item])
            try:    # Can it be converted to int?
                self.pardict[item] = int(self.pardict[item])
            except ValueError:
                try:   # No, but can it be converted to float?
                    self.pardict[item] = float(self.pardict[item])
                except ValueError:
                    pass
            if self.pardict[item] in self.boolstr_t:    # cast switches to boolean values
                self.pardict[item] = True
            elif self.pardict[item] in self.boolstr_f:
                self.pardict[item] = False
        # generate class attributes from all valid parameters
        # for k,v in self.pardict.iteritems():
        #    if len(k.split())==1 and k[0]!='!':vars(self)[k]=v

    def Write_Pars(self, path):
        """ Take the dict and write a GENE parameters file """
        parfileout = open(path, "w")
        specflag = 0
        for item in self.namelists:
            specflag = 0
            if item[0:-1] == 'species':
                specflag = 1
                parfileout.write('&'+item[0:-1]+'\n')
            else:
                parfileout.write('&'+item+'\n')
            if specflag:
                [parfileout.write(par[0:-1]+'='+self.pardict[par]+'\n')
                 for par in self.pardict.keys() if self.nmldict[par] == item]
            else:
                [parfileout.write(par+'='+self.pardict[par]+'\n')
                 for par in self.pardict.keys() if self.nmldict[par] == item]
            parfileout.write('/\n\n')

    def asnamedtuple(self):
        """ Return Parameters as a named tuple for easier usage """
        # We have to ignore parameters with whitespace (in info nml)
        valid = OrderedDict()
        for item in self.pardict:
            if " " not in item:
                valid[item] = self.pardict[item]
        return namedtuple('ParTuple', valid.keys())(*valid.values())



import numpy as np
import matplotlib.pyplot as plt
from scipy import linalg
import sys

def read_geometry_local(file_name):
    f = open(file_name,'r')
    file_raw = f.read()
    file_lines = file_raw.split('\n')

    parameters = {}
    l = 1
    while '/' not in file_lines[l] and len(file_lines[l])>0:
        lsplit = file_lines[l].split('=')
    #    print lsplit[0].strip()
        if lsplit[0].strip() == 'gridpoints':
            parameters[lsplit[0].strip()] = int(float(lsplit[1].strip()))
        elif lsplit[0].strip() == 'magn_geometry':
            parameters[lsplit[0].strip()] = lsplit[1].strip()[1:-1]
        elif len(lsplit[0]) > 0:
            parameters[lsplit[0].strip()] = float(lsplit[1])
        l += 1
        #print "lsplit",lsplit
    
    #print parameters

    geometry = {}
    #1. ggxx(pi1,pj1,k) 
    geometry['ggxx'] = np.empty(0)
    #2. ggxy(pi1,pj1,k)
    geometry['ggxy'] = np.empty(0)
    #3. ggxz(pi1,pj1,k)
    geometry['ggxz'] = np.empty(0)
    #4. ggyy(pi1,pj1,k) 
    geometry['ggyy'] = np.empty(0)
    #5. ggyz(pi1,pj1,k)
    geometry['ggyz'] = np.empty(0)
    #6. ggzz(pi1,pj1,k)
    geometry['ggzz'] = np.empty(0)
    #7. gBfield(pi1,pj1,k) 
    geometry['gBfield'] = np.empty(0)
    #8. gdBdx(pi1,pj1,k)
    geometry['gdBdx'] = np.empty(0)
    #9. gdBdy(pi1,pj1,k)
    geometry['gdBdy'] = np.empty(0)
    #10. gdBdz(pi1,pj1,k)
    geometry['gdBdz'] = np.empty(0)
    #11. gjacobian(pi1,pj1,k)
    geometry['gjacobian'] = np.empty(0)
    #12. gl_R(pi1,k)
    geometry['gl_R'] = np.empty(0)
    #13. gl_phi(pi1,k)
    geometry['gl_phi'] = np.empty(0)
    #14. gl_z(pi1,k)
    geometry['gl_z'] = np.empty(0)
    #15. gl_dxdR(pi1,k)
    geometry['gl_dxdR'] = np.empty(0)
    #16. gl_dxdZ(pi1,k)
    geometry['gl_dxdZ'] = np.empty(0)

    if 'sign_Ip_CW' in file_raw: 
        l += 4
    else:
        l += 1
    while file_lines[l]:
        line = file_lines[l].split()
        geometry['ggxx'] = np.append(geometry['ggxx'],float(line[0].strip()))
        geometry['ggxy'] = np.append(geometry['ggxy'],float(line[1].strip()))
        geometry['ggxz'] = np.append(geometry['ggxz'],float(line[2].strip()))
        geometry['ggyy'] = np.append(geometry['ggyy'],float(line[3].strip()))
        geometry['ggyz'] = np.append(geometry['ggyz'],float(line[4].strip()))
        geometry['ggzz'] = np.append(geometry['ggzz'],float(line[5].strip()))
        geometry['gBfield'] = np.append(geometry['gBfield'],float(line[6].strip()))
        geometry['gdBdx'] = np.append(geometry['gdBdx'],float(line[7].strip()))
        geometry['gdBdy'] = np.append(geometry['gdBdy'],float(line[8].strip()))
        geometry['gdBdz'] = np.append(geometry['gdBdz'],float(line[9].strip()))
        geometry['gjacobian'] = np.append(geometry['gjacobian'],float(line[10].strip()))
        geometry['gl_R'] = np.append(geometry['gl_R'],float(line[11].strip()))
        geometry['gl_phi'] = np.append(geometry['gl_phi'],float(line[12].strip()))
        geometry['gl_z'] = np.append(geometry['gl_z'],float(line[13].strip()))
        geometry['gl_dxdR'] = np.append(geometry['gl_dxdR'],float(line[14].strip()))
        geometry['gl_dxdZ'] = np.append(geometry['gl_dxdZ'],float(line[15].strip()))
        #print "l",l,float(line[15])
        l += 1
        
    
    #for i in geometry:
    #    plt.title(i)
    #    plt.plot(geometry[i])
    #    plt.show()    

    return parameters, geometry

def read_geometry_global(file_name):
    f = open(file_name,'r')
    file_raw = f.read()
    file_lines = file_raw.split('\n')

    parameters = {}
    #for i in range(11):
    #    lsplit = file_lines[i+1].split('=')
    #    if 'magn' in file_lines[i+1]:
    #        parameters[lsplit[0].strip()] = lsplit[1].strip()[1:-1]
    #    else:
    #        parameters[lsplit[0].strip()] = float(lsplit[1].strip())
    #    print parameters[lsplit[0].strip()]

    l=1
    while '/' not in file_lines[l] and len(file_lines[l])>0:
        lsplit = file_lines[l].split('=')
        #print lsplit[0],lsplit[1]
        if lsplit[0].strip() == 'gridpoints':
            parameters[lsplit[0].strip()] = int(float(lsplit[1].strip()))
        elif lsplit[0].strip() == 'magn_geometry':
            parameters[lsplit[0].strip()] = lsplit[1].strip()[1:-1]
        else:
            parameters[lsplit[0].strip()] = float(lsplit[1])
        l += 1
        print(parameters[lsplit[0].strip()])


    #lsplit = file_lines[11].split('=')
    #parameters[lsplit[0].strip()] = lsplit[1].strip()[1:-1]
    
    print(parameters)
    
    geometry = {}
    geometry['q'] = np.empty(0)
    geometry['gxx'] = np.empty(0)
    geometry['gxy'] = np.empty(0)
    geometry['gxz'] = np.empty(0)
    geometry['gyy'] = np.empty(0)
    geometry['gyz'] = np.empty(0)
    geometry['gzz'] = np.empty(0)
    geometry['Bfield'] = np.empty(0)
    geometry['dBdx'] = np.empty(0)
    geometry['dBdy'] = np.empty(0)
    geometry['dBdz'] = np.empty(0)
    geometry['jacobian'] = np.empty(0)
    geometry['C_y'] = np.empty(0)
    geometry['C_xy'] = np.empty(0)
    geometry['geo_R'] = np.empty(0)
    geometry['geo_Z'] = np.empty(0)
    geometry['geo_c1'] = np.empty(0)
    geometry['geo_c2'] = np.empty(0)
    geometry['dpdx_pm_arr'] = np.empty(0)

    for ln in range(len(file_lines)):
        if file_lines[ln].strip() in geometry:
            this_var = file_lines[ln].strip()
            print(this_var)
            ln2 = ln+1
            this_line = file_lines[ln2]
            while not this_line.strip() in geometry:
                lsplit = this_line.split()
                for i in lsplit:
                    geometry[this_var] = np.append(geometry[this_var],float(i)) 
                ln2 += 1
                if ln2 != len(file_lines):
                    this_line = file_lines[ln2]
                else:
                    #Need to trigger exit
                    this_line = 'q'

    nx0 = int(len(geometry['q']))
    nz0 = int(parameters['gridpoints'])
    #plt.plot(geometry['gxx'])
    #plt.show()
    geometry['gxx']=geometry['gxx'].reshape((nz0,nx0))
    geometry['gxy']=geometry['gxy'].reshape((nz0,nx0))
    geometry['gxz']=geometry['gxz'].reshape((nz0,nx0))
    geometry['gyy']=geometry['gyy'].reshape((nz0,nx0))
    geometry['gyz']=geometry['gyz'].reshape((nz0,nx0))
    geometry['gzz']=geometry['gzz'].reshape((nz0,nx0))
    geometry['Bfield']=geometry['Bfield'].reshape((nz0,nx0))
    geometry['dBdx']=geometry['dBdx'].reshape((nz0,nx0))
    geometry['dBdy']=geometry['dBdy'].reshape((nz0,nx0))
    geometry['dBdz']=geometry['dBdz'].reshape((nz0,nx0))
    geometry['jacobian']=geometry['jacobian'].reshape((nz0,nx0))
    geometry['geo_R']=geometry['geo_R'].reshape((nz0,nx0))
    geometry['geo_Z']=geometry['geo_Z'].reshape((nz0,nx0))
    geometry['geo_c1']=geometry['geo_c1'].reshape((nz0,nx0))
    geometry['geo_c2']=geometry['geo_c2'].reshape((nz0,nx0))

    return parameters, geometry

def write_tracer_efit_file(parameters,geometry,file_name):
    f = open(file_name,'w')
    f.write('&parameters\n')
    f.write('gridpoints =    '+str(int(float(parameters['gridpoints'])))+'\n')
    f.write('q0 =    '+str(parameters['q0'])+'\n')
    f.write('shat =    '+str(parameters['shat'])+'\n')
    f.write('s0 =    '+str(parameters['s0'])+'\n')
    f.write('minor_r =    '+str(parameters['minor_r'])+'\n')
    f.write('major_R =    '+str(parameters['major_R'])+'\n')
    f.write('trpeps =    '+str(parameters['trpeps'])+'\n')
    f.write('beta =    '+str(parameters['beta'])+'\n')
    f.write('Lref =    '+str(parameters['Lref'])+'\n')
    f.write('Bref =    '+str(parameters['Bref'])+'\n')
    f.write('magn_geometry =    '+'\''+str(parameters['magn_geometry'])+'\'\n/\n')

    nz0 = int(float(parameters['gridpoints']))
    gsize = np.shape(geometry['gxx'])    
    nx0 = gsize[0]*gsize[1]/nz0

    #geometry['q'] = np.empty(0)
    f.write('q\n')
    for i in range(nx0):
        f.write("%20.12E"% geometry['q'][i])
        if (i+1) % 3 == 0:
            f.write('\n')

    if len(geometry['q']) % 3 != 0:
        f.write('\n')

    #geometry['gxx'] = np.empty(0)
    temp = np.reshape(geometry['gxx'],nx0*nz0)
    f.write('gxx\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    #geometry['gxy'] = np.empty(0)
    temp = np.reshape(geometry['gxy'],nx0*nz0)
    f.write('gxy\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['gxz'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['gxz'],nx0*nz0)
    f.write('gxz\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['gyy'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['gyy'],nx0*nz0)
    f.write('gyy\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    #geometry['gyz'] = np.empty(0)
    temp = np.reshape(geometry['gyz'],nx0*nz0)
    f.write('gyz\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['gzz'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['gzz'],nx0*nz0)
    f.write('gzz\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['Bfield'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['Bfield'],nx0*nz0)
    f.write('Bfield\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['dBdx'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['dBdx'],nx0*nz0)
    f.write('dBdx\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['dBdy'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['dBdy'],nx0*nz0)
    f.write('dBdy\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['dBdz'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['dBdz'],nx0*nz0)
    f.write('dBdz\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['jacobian'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['jacobian'],nx0*nz0)
    f.write('jacobian\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['C_y'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    f.write('C_y\n')
    for i in range(len(geometry['C_y'])):
        f.write("%20.12E"% geometry['C_y'][i])
        if (i+1) % 3 == 0:
            f.write('\n')
    #geometry['C_xy'] = np.empty(0)

    if len(geometry['C_y']) % 3 != 0:
        f.write('\n')

    f.write('C_xy\n')
    for i in range(len(geometry['C_xy'])):
        f.write("%20.12E"% geometry['C_xy'][i])
        if (i+1) % 3 == 0:
            f.write('\n')

    if len(geometry['C_xy']) % 3 != 0:
        f.write('\n')

    #geometry['geo_R'] = np.empty(0)
    temp = np.reshape(geometry['geo_R'],nx0*nz0)
    f.write('geo_R\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['geo_Z'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['geo_Z'],nx0*nz0)
    f.write('geo_Z\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['geo_c1'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['geo_c1'],nx0*nz0)
    f.write('geo_c1\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['geo_c2'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    temp = np.reshape(geometry['geo_c2'],nx0*nz0)
    f.write('geo_c2\n')
    for i in range(len(temp)):
        f.write("%20.12E"% temp[i])
        if (i+1) % 16 == 0:
            f.write('\n')
    #geometry['dpdx_pm_arr'] = np.empty(0)
    if nx0*nz0 % 16 != 0:
        f.write('\n')
    f.write('dpdx_pm_arr\n')
    for i in range(len(geometry['dpdx_pm_arr'])):
        f.write("%20.12E"% geometry['dpdx_pm_arr'][i])
        if (i+1) % 3 == 0:
            f.write('\n')
    f.close()





import csv
import numpy as np
import math
import matplotlib.pyplot as plt

from .max_pedestal_finder import find_pedestal
from .read_iterdb_file import read_iterdb_file
from .read_pfile import p_to_iterdb_format
from .read_EFIT import read_EFIT
from .read_write_geometry import read_geometry_global
from .parIOWrapper import init_read_parameters_file
#from max_profile_reader import profile_e_info
#from max_profile_reader import profile_i_info
from .interp import interp
#Created by Max T. Curie  11/02/2020
#Last edited by Max Curie 11/02/2020
#Supported by scripts in IFS

def input():
    temp=input("The profile file type: 1. ITERDB  2. pfile")
    suffix="1"
    if temp==1:
        profile_type="ITERDB"    # "ITERDB" "pfile"  "GENE"
    elif temp==2:
        profile_type="pfile"    # "ITERDB" "pfile"  "GENE"
    else:
        print("Please type 1 or 2")

    profile_name = input("The profile file name: ")


    temp=input("The geometry file type: 1. gfile  2. GENE_tracor")
    if temp==1:
        geomfile_type="gfile"    
    elif temp==2:
        geomfile_type="GENE_tracor"  
        suffix=input("Suffix (0001, 1, dat): ")
    else:
        print("Please type 1 or 2")

    geomfile_name = input("The geometry file name: ")   

    return profile_type, geomfile_type, profile_name, geomfile_name, suffix
    

def read_profile_file(profile_type,profile_name,geomfile_name,suffix='.dat'):
    if profile_type=="ITERDB":
        rhot0, te0, ti0, ne0, ni0, nz0, vrot0 = read_iterdb_file(profile_name)
        psi0 = np.linspace(0.,1.,len(rhot0))
        rhop0 = np.sqrt(np.array(psi0))
        return rhot0, rhop0, te0, ti0, ne0, ni0, nz0, vrot0

    elif profile_type=="pfile":
        rhot0, rhop0, te0, ti0, ne0, ni0, nz0, vrot0 = p_to_iterdb_format(profile_name,geomfile_name)
        return rhot0, rhop0, te0, ti0, ne0, ni0, nz0, vrot0

    elif profile_type=="profile_e":
        if suffix=="dat":
            suffix=".dat"
        else:
            suffix="_"+suffix
        rhot0, te0, ti0, ne0, ni0, nz0, vrot0 = read_iterdb_file(profile_name)
        psi0 = np.linspace(0.,1.,len(rhot0))
        rhop0 = np.sqrt(np.array(psi0))

        x_a,x_rho_ref,T,n0,omt,omn = profile_e_info(suffix)
        if 1==0:
            plt.clf()
            plt.plot(x_a,n0*1.0e19,label='profile')
            plt.plot(rhot0,ne0,label='ITERDB')
            #plt.plot(x_rho_ref,T,label='x_rho_ref')
            plt.legend()
            plt.show()

            plt.clf()
            plt.plot(x_a,T*1000.,label='profile')
            plt.plot(rhot0,te0,label='ITERDB')
            #plt.plot(x_rho_ref,T,label='x_rho_ref')
            plt.legend()
            plt.show()
        #x_a,x_rho_ref,T,n0,omt,omn = profile_i_info(suffix)
        
        rhot0_range_min=np.argmin(abs(rhot0-x_a[0]))
        rhot0_range_max=np.argmin(abs(rhot0-x_a[-1]))
        rhot=rhot0[rhot0_range_min:rhot0_range_max]
        rhop=rhop0[rhot0_range_min:rhot0_range_max]
        ti=ti0[rhot0_range_min:rhot0_range_max]
        ni=ni0[rhot0_range_min:rhot0_range_max]
        vrot=vrot0[rhot0_range_min:rhot0_range_max]

        rhot0 = np.linspace(min(rhot),max(rhot),len(rhot)*3.)
        
        rhop0=interp(rhot,rhop,rhot0)
        te0 = interp(x_a,T*1000.,rhot0)
        ti0=interp(rhot,ti,rhot0)
        ni0=interp(rhot,ni,rhot0)
        nz0 = interp(rhot,nz,rhot0)
        ne0 = interp(x_a,n0*1.0e19,rhot0)

        vrot0=interp(rhot,vrot,rhot0)
        
        print("**************************")
        print("**************************")
        print("x_a min max: "+str(np.min(x_a))+", "+str(np.max(x_a)))
        print("**************************")
        print("**************************")
        print("**************************")

        return rhot0, rhop0, te0, ti0, ne0, ni0, vrot0

def read_geom_file(file_type,file_name,suffix="dat"):
    if file_type=="gfile":
        EFITdict = read_EFIT(file_name)
        
        xgrid = EFITdict['rhotn'] #rho_tor

        q = EFITdict['qpsi']
        R = EFITdict['R']
        print("**************************")
        print("**************************")
        print(str(R))
        print(np.shape(R))
        R_ref=R[int(len(R)/2)]
        print("**************************")
        print("**************************")
        return xgrid, q, R_ref
        
    elif file_type=="GENE_tracor":
        gpars,geometry = read_geometry_global(file_name)
        q=geometry['q']
        R=geometry['geo_R']
        print("**************************")
        print("**************************")
        print(str(R))
        print(np.shape(R))
        (nz0,nx0)=np.shape(R)
        R_ref=R[int(nz0/2),int(nx0/2)]
        print("**************************")
        print("**************************")
        
        if suffix=="dat":
            suffix=".dat"
        else:
            suffix="_"+suffix

        pars = init_read_parameters_file(suffix)
        Lref=pars['Lref']

        Bref=pars['Bref']
        x0_from_para=pars['x0']
        #xgrid=rho_tor
        if 'lx_a' in pars:
            xgrid = np.arange(pars['nx0'])/float(pars['nx0']-1)*pars['lx_a']+pars['x0']-pars['lx_a']/2.0
        else:
            xgrid = np.arange(pars['nx0'])/float(pars['nx0']-1)*pars['lx'] - pars['lx']/2.0
        print("**************************")
        print("**************************")
        print("xgrid min max: "+str(np.min(xgrid))+", "+str(np.max(xgrid)))
        print("**************************")
        print("**************************")
        print("**************************")
        return xgrid, q, Lref, R_ref, Bref, x0_from_para
    
def profile_e_info(suffix):
    gene_e = 'profiles_e'+suffix
    gene_i = 'profiles_i'+suffix
    gene_z = 'profiles_z'+suffix
    suffix=gene_e
    f = open(suffix, 'r')
    #prof=f.read()#the read from the profile
    prof = np.genfromtxt(suffix, dtype=float, skip_header=2)
    x_a=prof[:,0]
    x_rho_ref=prof[:,1]
    T=prof[:,2]
    n0=prof[:,3]
    omt=prof[:,4]
    omn=prof[:,5]

    return x_a,x_rho_ref,T,n0,omt,omn 

def profile_i_info(suffix):
    gene_e = 'profiles_e'+suffix
    gene_i = 'profiles_i'+suffix
    gene_z = 'profiles_z'+suffix
    suffix=gene_i
    f = open(suffix, 'r')
    #prof=f.read()#the read from the profile
    prof = np.genfromtxt(suffix, dtype=float, skip_header=2)
    x_a=prof[:,0]
    x_rho_ref=prof[:,1]
    T=prof[:,2]
    n0=prof[:,3]
    omt=prof[:,4]
    omn=prof[:,5]

    return x_a,x_rho_ref,T,n0,omt,omn 

def profile_z_info(suffix):
    gene_e = 'profiles_e'+suffix
    gene_i = 'profiles_i'+suffix
    gene_z = 'profiles_z'+suffix
    suffix=gene_z
    f = open(suffix, 'r')
    #prof=f.read()#the read from the profile
    prof = np.genfromtxt(suffix, dtype=float, skip_header=2)
    x_a=prof[:,0]
    x_rho_ref=prof[:,1]
    T=prof[:,2]
    n0=prof[:,3]
    omt=prof[:,4]
    omn=prof[:,5]

    return x_a,x_rho_ref,T,n0,omt,omn 


import tensorflow as tf  
import pandas as pd 
import numpy as np 

def load_data(filename_list):
    #*******start of loading data*******************
    for i in range(len(filename_list)):
        filename=filename_list[i]
        df=pd.read_csv(filename)
        df=df.dropna()
        try:
            df=df.drop(columns=['change'])
        except:
            pass

        #merge the dataframe
        if i==0:
            df_merge=df
        elif i!=0:
            df_merge=pd.concat([df_merge, df], axis=0)
            
    return df_merge
def find_data_range(para_list_min,para_list_max):
    para_name_list=['nu', 'zeff', 'eta', 'shat',  'beta',  'ky',   'mu', 'xstar']
    query_name_str=''

    for i in range(len(para_name_list)):
        if i==0:
            query_name_str=query_name_str+\
                            str(para_list_min[i])+'<'+para_name_list[i]+\
                            ' and '+\
                            para_name_list[i]+'<'+str(para_list_max[i])
        else:
            query_name_str=query_name_str+' and '+\
                            str(para_list_min[i])+'<'+para_name_list[i]+\
                            ' and '+\
                            para_name_list[i]+'<'+str(para_list_max[i])

    return query_name_str



#This script is intended to find the top and the mid pedestal of the H mod plasma profile for the pre and post processing of the simulation

#Developed by Max Curie on 01/22/2020

import numpy as np
import matplotlib.pyplot as plt
from .efittools import read_efit_file

def find_pedestal(file_name, path_name, plot):
    if len(path_name)==0 and ('/' not in file_name):
       path_name = './'
    eqdskdata=read_efit_file(path_name+file_name)

    p  = eqdskdata['pressure'] #pressure read from eqdsk
    x  = eqdskdata['rhotor']   #radial location read from eqdsk
    dp = eqdskdata['pprime']   #First order of pressure
    
    x=x[int(len(x)*0.6):]
    p=p[int(len(p)*0.6):]
    dp=dp[int(len(dp)*0.6):]

    dp0   = np.gradient(p,x)  #Second order of pressure
    ddp   = np.gradient(dp,x)  #Second order of pressure

    midped = x[np.argmin(dp)]
    topped = x[np.argmin(ddp)]

    print(topped)
    print(midped)
    
    if plot == True:
        plt.clf()
        plt.title(r'$Compare$')
        plt.xlabel(r'$r/a$',fontsize=10)
        plt.ylabel(r'$ab$',fontsize=13)
        plt.plot(x,p/np.max(abs(p)),label="p")
        plt.plot(x,-dp/np.max(abs(dp)),label="dp_from_EFIT")
        plt.plot(x,-dp0/np.max(abs(dp0)),label="dp")
        plt.plot(x,ddp/np.max(abs(ddp)),label="ddp")
        plt.axvline(x=midped, label="Mid-pedestal", color="red")
        plt.axvline(x=topped, label="Top-pedestal", color="green")
        plt.legend()
        #plt.savefig('compare.png')
        plt.show()

    return midped, topped

def find_pedestal_from_data(uni_rhot,p,dp,plot=False):
    ddp   = np.gradient(dp,uni_rhot)  #Second order of pressure
    midped = uni_rhot[np.argmin(dp)]
    topped = uni_rhot[np.argmin(ddp)]
    if plot == 1:
        plt.clf()
        plt.title(r'$Compare$')
        plt.xlabel(r'$r/a$',fontsize=10)
        plt.ylabel(r'$ab$',fontsize=13)
        #plt.plot(uni_rhot,p,label="p")
        plt.plot(uni_rhot,dp,label="dp")
        plt.plot(uni_rhot,ddp,label="ddp")
        plt.axvline(x=midped, label="Mid-pedestal", color="red")
        plt.axvline(x=topped, label="Top-pedestal", color="green")
        plt.legend()
        #plt.savefig('compare.png')
        plt.show()
    print(topped)
    print(midped)
    return midped, topped

